@@ -1,7 +1,9 @@
 /******************************************************************
+ *
+ * $Id: open2300db2APRS.c,v 1.11 2012/11/01 18:57:18 we7u Exp $
  *
  * Copyright (C) 2005 Jim Chandler <jim@n0vh.org>
- * Portions Copyright (C) 2004-2005 The Xastir Group
+ * Portions Copyright (C) 2004-2012 The Xastir Group
  *
  * Portions copied from Bruce Bennett's excellent Davis WX work
  *
@@ -43,6 +45,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -64,6 +67,7 @@
 #define VALID_RAIN24H   0x20
 #define VALID_HUMIDITY  0x40
 #define VALID_AIRPRESS  0x80
+#define VALID_RAINTOT   0x100
 
 #define MTPS2MPH        2.2369
 #define DEGC2DEGF       1.8
@@ -172,6 +176,7 @@ int APRS_str(char *APRS_buf,
             double temp,
             double rain1hr,
             double rain24h,
+            double raintot,
             double humidity,
             double airpressure,
             unsigned int valid_data_flgs,
@@ -360,6 +365,7 @@ int APRS_str(char *APRS_buf,
     }
     strcat(APRS_buf,pbuf);
 
+
     if(valid_data_flgs & VALID_HUMIDITY) {
         intval = (humidity + 0.5); // rounding to whole percent
         if (intval > 100) { // Unlike the space shuttle engines, 100 % is max
@@ -414,6 +420,43 @@ int APRS_str(char *APRS_buf,
 
     }
     strcat(APRS_buf,pbuf);
+
+    // NOW THIS MAKES THE STRING NO LONGER A VALID APRS WX REPORT, but
+    // we don't care:  Xastir does NOT just transmit this string, it parses it
+    // and re-recreates the correct string to transmit.  We do this because
+    // APRS doesn't have a "total rain" string in its weather report, but
+    // Xastir likes to have that value around.
+    if(valid_data_flgs & VALID_RAINTOT) {
+        if (Metric_Data)
+            intval = ((raintot)*MM2IN100TH + 0.5); // converting & rounding to whole 1/100 inch
+        else
+            intval = (raintot*100.0 + 0.5); // rounding to whole 1/100 inch
+
+        // Can't handle greater than 99.99 inches of total rain 
+        if (intval > 9999) { 
+            if (debug_level & 1)
+                fprintf(stderr,"err: total Rainfall  > 99.99 inch - reporting 9.99 inches\n");
+            sprintf(pbuf, "T9999");
+
+        } 
+        else if (intval < -99) {
+            if (debug_level & 1)
+                fprintf(stderr,"err: total Rainfall negative\n");
+            sprintf(pbuf, "\0\0\0\0\0");
+
+        } 
+        else {
+            sprintf(pbuf, "T%0.4d", intval);
+        }
+    } 
+    else {
+        if (debug_level & 1)
+            fprintf(stderr,"info: total Rainfall flagged as invalid\n");
+        sprintf(pbuf, "\0\0\0\0\0");
+
+    }
+    strcat(APRS_buf,pbuf);
+
     strcat(APRS_buf,"xDvs\n");  // add X aprs and LaCrosse WX station ID's and <lf>
 
     if (debug_level & 1)
@@ -437,6 +480,7 @@ int Get_Latest_WX( double *winddir,
                 double *temp,
                 double *rain1hr,
                 double *rain24h,
+                double *raintot,
                 double *humidity,
                 double *airpressure,
                 unsigned int *valid_data_flgs,
@@ -447,6 +491,7 @@ int Get_Latest_WX( double *winddir,
     char latest_timestamp[20];
     char query_buffer[160];
     int nrows, row_cnt, item_count;
+    int nfields;
 
 		
     // Find latest, see if it's new to us
@@ -499,7 +544,7 @@ int Get_Latest_WX( double *winddir,
     // release query buffer
     mysql_free_result(result);
 
-    sprintf(query_buffer,"SELECT wind_angle, windspeed, temp_out, rain_1h, rain_24h, rel_hum_out, rel_pressure FROM weather WHERE timestamp = %s", last_timestamp);
+    sprintf(query_buffer,"SELECT wind_angle, windspeed, temp_out, rain_1h, rain_24h, rel_hum_out, rel_pressure, rain_total FROM weather WHERE timestamp = %s", last_timestamp);
 
     if (mysql_query(&mysql, query_buffer)) {
         if (debug_level & 1)
@@ -521,8 +566,10 @@ int Get_Latest_WX( double *winddir,
         return 0;
     }
     else {
+        nfields=mysql_num_fields(result);
+        row=mysql_fetch_row(result);
         if (debug_level & 1)
-            fprintf(stderr,"info: Latest Weather Data query: number of types of readings %d\n",nrows);
+            fprintf(stderr,"info: Latest Weather Data query: number of types of readings %d\n",nfields);
     }
 
     *valid_data_flgs = 0;
@@ -577,6 +624,12 @@ int Get_Latest_WX( double *winddir,
     item_count++;
     if(debug_level & 1)
          fprintf(stderr,"air pressure %f\n ",*airpressure);
+    //case  RAIN_TOTAL
+    *raintot = strtod(row[7],NULL);
+    *valid_data_flgs |= VALID_RAINTOT;  
+    item_count++;
+    if(debug_level & 1)
+         fprintf(stderr,"rainfall since reset %f\n ",*raintot);
 
     *Metric_Data = 0;  // My station reports F, knots and inHG
     // release query buffer	& close connection
@@ -649,6 +702,7 @@ int main(int argc, char **argv)
     double temp;
     double rain1hr;
     double rain24h;
+    double raintot;
     double humidity;
     double airpressure;
     unsigned int valid_data_flgs;
@@ -809,12 +863,14 @@ int main(int argc, char **argv)
         if(!(dly_cnt--)) {	 
             dly_cnt = 25;		// Every 'dly_cnt' passes check for WX data update
             if((dsts = Get_Latest_WX(&winddir,&windspeed,&windgust,
-                &temp,&rain1hr,&rain24h,&humidity,&airpressure,
-                &valid_data_flgs,&Metric_Data)) !=0 ) {
+                                     &temp,&rain1hr,&rain24h,&raintot,
+                                     &humidity,&airpressure,
+                                     &valid_data_flgs,&Metric_Data)) !=0 ) {
                 if ( dsts > 0 ) {
                     data_len = APRS_str(WX_APRS, winddir,windspeed,windgust,
-                        temp, rain1hr, rain24h, humidity, airpressure,
-                        valid_data_flgs, Metric_Data);
+                                        temp, rain1hr, rain24h, raintot, 
+                                        humidity, airpressure,
+                                        valid_data_flgs, Metric_Data);
 
                     if (!data_len) {
                         if (debug_level & 1)
