@@ -1,9 +1,9 @@
 /*
- * $Id: bulletin_gui.c,v 1.37 2006/04/24 19:21:41 we7u Exp $
+ * $Id: bulletin_gui.c,v 1.46 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2006  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -80,7 +80,7 @@
 #include "leak_detection.h"
 
 
-
+extern XmFontList fontlist1;    // Menu/System fontlist
 Widget Display_bulletins_dialog = NULL;
 Widget Display_bulletins_text = NULL;
 Widget dist_data = NULL;
@@ -133,7 +133,7 @@ void bulletin_message(char *call_sign, char *tag, char *packet_message, time_t s
     char temp_my_course[10];
     char temp_text[30];
     double distance;
-    XmTextPosition pos, eol;
+    XmTextPosition pos, eol, eod;
     struct tm *tmp;
     time_t timehd;
     char time_str[20];
@@ -164,8 +164,9 @@ void bulletin_message(char *call_sign, char *tag, char *packet_message, time_t s
 
 begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:bulletin_message" );
 
-        if ((Display_bulletins_dialog != NULL)) {   // Dialog is up
+        if ((Display_bulletins_dialog != NULL) && Display_bulletins_text != NULL) {   // Dialog is up
 
+            eod = XmTextGetLastPosition(Display_bulletins_text);
             xastir_snprintf(temp_text,
                 sizeof(temp_text),
                 "%s",
@@ -177,21 +178,34 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:bulletin_
             if (XmTextFindString(Display_bulletins_text, 0, temp_text, XmTEXT_FORWARD, &pos)) {
 
                 // Found it, so now find the end-of-line for it
-                (void)XmTextFindString(Display_bulletins_text, pos, "\n", XmTEXT_FORWARD, &eol);
-                eol++;
+                if (XmTextFindString(Display_bulletins_text, pos, "\n", XmTEXT_FORWARD, &eol))
+                  eol++;
+                else 
+                  eol = eod;
 
                 // And replace the old bulletin with a new copy
+                if (eol == eod)
+                  temp[strlen(temp)-1] = '\0';
                 XmTextReplace(Display_bulletins_text, pos, eol, temp);
             } else {
-                for (pos = 0; strlen(temp_text) > 12 && pos < XmTextGetLastPosition(Display_bulletins_text);) {
+                for (pos = 0; strlen(temp_text) > 12 && pos < eod;) {
                     if (XmCOPY_SUCCEEDED == XmTextGetSubstring(Display_bulletins_text, pos, 14, 30, temp_text)) {
                         if (temp_text[0] && strncmp(temp, temp_text, 14) < 0)
                             break;
                     } else
                         break;
 
-                    (void)XmTextFindString(Display_bulletins_text, pos, "\n", XmTEXT_FORWARD, &eol);
-                    pos = ++eol;
+                    if (XmTextFindString(Display_bulletins_text, pos, "\n", XmTEXT_FORWARD, &eol))
+                      pos = ++eol;
+                    else
+                      pos = eod;
+                }
+                if (pos == eod) {
+                  temp[strlen(temp)-1] = '\0'; // End-of-Data remove trailing LF
+                  if (pos > 0) { // Already have text. Need to insert LF between items
+                    memmove(&temp[1], temp, strlen(temp));
+                    temp[0] = '\n';
+                  }
                 }
                 XmTextInsert(Display_bulletins_text,pos,temp);
             }
@@ -237,12 +251,6 @@ void bulletin_data_add(char *call_sign, char *from_call, char *data,
         char *seq, char type, char from) {
     int distance = -1;
 
-    // Update the View->Bulletins dialog if it's up
-    bulletin_message(from_call,
-        call_sign,
-        data,
-        sec_now());
-
     // Add to the message database
     (void)msg_data_add(call_sign,
         from_call,
@@ -302,6 +310,12 @@ void bulletin_data_add(char *call_sign, char *from_call, char *data,
 //            fprintf(stderr,", but distance didn't work out!\n");
         }
     }
+    // Update the View->Bulletins dialog if it's up
+    bulletin_message(from_call,
+        call_sign,
+        data,
+        sec_now());
+
 }
 
 
@@ -599,6 +613,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
                 appshell,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
+                XmNfontList, fontlist1,
                 NULL);
 
         pane = XtVaCreateWidget("Bulletins pane",
@@ -629,6 +644,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
                 XmNrightAttachment, XmATTACH_NONE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         dist_data = XtVaCreateManagedWidget("dist_data", 
@@ -650,6 +666,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
                 XmNleftOffset, 10,
                 XmNrightAttachment,XmATTACH_NONE,
                 XmNnavigationType, XmTAB_GROUP,
+                XmNfontList, fontlist1,
                 NULL);
 
         dist_units = XtVaCreateManagedWidget((english_units?langcode("UNIOP00004"):langcode("UNIOP00005")),
@@ -664,6 +681,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
                 XmNrightAttachment, XmATTACH_NONE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         button_range = XtVaCreateManagedWidget(langcode("BULMW00003"),
@@ -679,6 +697,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
                 XmNnavigationType, XmTAB_GROUP,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         zero_bulletin_data = XtVaCreateManagedWidget(langcode("WPUPCFD029"),
@@ -694,6 +713,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
                 XmNnavigationType, XmTAB_GROUP,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
         XtAddCallback(zero_bulletin_data,XmNvalueChangedCallback,Zero_Bulletin_Data_toggle,"1");
         if (view_zero_distance_bulletins)
@@ -722,6 +742,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
         XtSetArg(args[n], XmNrightOffset, 5); n++;
         XtSetArg(args[n], XmNforeground, MY_FG_COLOR); n++;
         XtSetArg(args[n], XmNbackground, MY_BG_COLOR); n++;
+        XtSetArg(args[n], XmNfontList, fontlist1); n++;
 
 
         Display_bulletins_text = XmCreateScrolledText(form,
@@ -741,6 +762,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
                 XmNrightPosition, 3,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         XtAddCallback(button_range, XmNactivateCallback, Display_bulletins_change_range, Display_bulletins_dialog);
