@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: color.c,v 1.11 2005/01/08 10:06:52 we7u Exp $
+ * $Id: color.c,v 1.23 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -32,6 +35,10 @@
 #include "color.h"
 #include "xa_config.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 static color_load color_choice[MAX_COLORS];
 static int colors_loaded;
@@ -115,6 +122,7 @@ Pixel GetPixelByName( Widget w, char *colorname) {
 
     found=-1;
     i=0;
+
     do {
         if (strcmp(color_choice[i].colorname, colorname)==0)
             found=i;
@@ -123,20 +131,31 @@ Pixel GetPixelByName( Widget w, char *colorname) {
     } while (i<colors_loaded && found <0);
 
     if (found >= 0) {
-        if (XAllocColor(dpy,cmap,&color_choice[found].color))
+
+        // XFreeColors() here generates an error.  Why?
+        // "BadAccess (attempt to access private resource denied)"
+        // XFreeColors(dpy, cmap, &(color_choice[found].color.pixel),1,0);
+
+        if (XAllocColor(dpy,cmap,&color_choice[found].color)) {
             return(color_choice[found].color.pixel);
+        }
         else {
             xastir_snprintf(warning, sizeof(warning), "Couldn't allocate color %s", colorname);
             XtWarning(warning);
             return(BlackPixel(dpy,scr));
         }
-    } else {
+    }
+    else {
         xastir_snprintf(warning, sizeof(warning), "Couldn't find color %s", colorname);
         XtWarning(warning);
         return(BlackPixel(dpy,scr));
     }
 }
 
+
+
+
+
 void setup_visual_info(Display* dpy, int scr) {
     int visuals_matched, i, j;
     XVisualInfo *visual_list, *vp;
