@@ -1,9 +1,9 @@
 /*
- * $Id: database.h,v 1.17 2005/06/09 05:08:09 tvrusso Exp $
+ * $Id: database.h,v 1.56 2012/11/01 18:57:19 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,9 @@
  * Look at the README for more information on the program.
  */
 
+/* Note: this file should be called db.h, but was renamed to database.h
+ * to avoid conflicts with the Berkeley DB package.  */
+
 /*
  * Database structures
  *
@@ -33,13 +36,10 @@
 
 #define MSG_INCREMENT 200
 #define MAX_CALLSIGN 9       // Objects are up to 9 chars
-#define MAX_TACTICAL_CALL 20 // Up to XX chars for tactical calls
+#define MAX_TACTICAL_CALL 57 // Up to XX chars for tactical calls
 #define MAX_COMMENT_LINES 20  // Save XX unique comment strings per station
 #define MAX_STATUS_LINES 20   // Save XX unique status strings per station
 
-/* define max tnc line size (should be from tnc.h) */
-#define MAX_TNC_LINE_SIZE 300
-
 /* define max size of info field */
 #define MAX_INFO_FIELD_SIZE 256
 
@@ -82,7 +82,8 @@ enum APRS_Types {
     APRS_MICE,
     APRS_GRID,
     DALLAS_ONE_WIRE,
-    DAVISMETEO
+    DAVISMETEO,
+    DAVISAPRSDL
 };
 
 
@@ -94,6 +95,12 @@ enum APRS_Types {
 #define NORMAL_GPS_RMC  'C'
 #define NORMAL_GPS_GGA  'A'
 #define NORMAL_GPS_GLL  'L'
+#define APRS_WX1        '1'
+#define APRS_WX2        '2'
+#define APRS_WX3        '3'
+#define APRS_WX4        '4'
+#define APRS_WX5        '5'
+#define APRS_WX6        '6'
 
 /* define RECORD ACTIVES */
 #define RECORD_ACTIVE    'A'
@@ -105,6 +112,7 @@ enum APRS_Types {
 #define DATA_VIA_TNC   'T'
 #define DATA_VIA_NET   'I'
 #define DATA_VIA_FILE  'F'
+#define DATA_VIA_DATABASE  'D'
 
 
 /* define Heard info type */
@@ -133,6 +141,9 @@ enum APRS_Types {
 #define MAX_MESSAGE_LENGTH  100
 #define MAX_MESSAGE_ORDER    10
 
+// track export file formats
+#define EXPORT_XASTIR_TRACK 0
+#define EXPORT_KML_TRACK 1
 
 extern char *get_most_recent_ack(char *callsign);
  
@@ -271,6 +282,11 @@ typedef struct {                //                      strlen
     char    wx_fuel_moisture[5];// in %                     2
     char    wx_type;
     char    wx_station[MAX_WXSTATION];
+    int     wx_compute_rain_rates;  //  Some stations provide rain rates
+                                  // directly, others require Xastir to
+                                  // compute from total rain.  Flag this,
+                                  // so we don't clobber useful info from
+                                  // a station.
 } WeatherRow;
 
 
@@ -337,15 +353,14 @@ typedef struct _MultipointRow{
 //
 // todo: check the string length!
 //
-
 typedef struct _DataRow {
 
     struct _DataRow *n_next;    // pointer to next element in name ordered list
     struct _DataRow *n_prev;    // pointer to previous element in name ordered
                                 // list
-    struct _DataRow *t_next;    // pointer to next element in time ordered
+    struct _DataRow *t_newer;   // pointer to next element in time ordered
                                 // list (newer)
-    struct _DataRow *t_prev;    // pointer to previous element in time ordered
+    struct _DataRow *t_older;   // pointer to previous element in time ordered
                                 // list (older)
 
     char call_sign[MAX_CALLSIGN+1]; // call sign or name index or object/item
@@ -372,6 +387,19 @@ typedef struct _DataRow {
     short flag;                 // several flags, see below
     char pos_amb;               // Position ambiguity, 0 = none,
                                 // 1 = 0.1 minute...
+
+    unsigned int error_ellipse_radius; // Degrades precision for this
+                                // station, from 0 to 65535 cm or
+                                // 655.35 meters.  Assigned when we
+                                // decode each type of packet.
+                                // Default is 6.0 meters (600 cm)
+                                // unless we know the GPS position
+                                // is augmented, or is degraded by
+                                // less precision in the packet.
+
+    unsigned int lat_precision;	// In 100ths of a second latitude
+    unsigned int lon_precision;	// In 100ths of a second longitude
+
     int trail_color;            // trail color (when assigned)
     char record_type;
     char data_via;              // L local, T TNC, I internet, F file
@@ -454,6 +482,8 @@ typedef struct _VerticeRow{
     struct  _VerticeRow *next;  // Pointer to next record in list
 } VerticeRow;
 
+#define CAD_LABEL_MAX_SIZE 40
+#define CAD_COMMENT_MAX_SIZE 256
 
 
 // CAD Objects
@@ -466,11 +496,15 @@ typedef struct _CADRow {
     int line_width;             // Border line width
     float computed_area;        // Area in square kilometers
     float raw_probability;      // Probability of area (POA) or probability of
-                                // detection (POD)
+                                // detection (POD) stored as probability 
+                                // with a value between 0 and 1.
+                                // Set and get with CAD_object_get_raw_probability()
+                                // and CAD_object_set_raw_probability(), rather 
+                                // than by a direct request for CADRow->raw_probability.
     long label_latitude;        // Latitude for label placement
     long label_longitude;       // Longitude for label placement
-    char label[40];             // Name of polygon
-    char comment[256];          // Comments associated with polygon
+    char label[CAD_LABEL_MAX_SIZE];             // Name of polygon
+    char comment[CAD_COMMENT_MAX_SIZE];          // Comments associated with polygon
 } CADRow;
 
 
@@ -478,7 +512,9 @@ extern CADRow *CAD_list_head;
 
 
 
-// station flag definitions
+// station flag definitions.  We have 16 bits available here as
+// "flag" in "DataRow" is defined as a short.
+//
 #define ST_OBJECT       0x01    // station is an object
 #define ST_ITEM         0x02    // station is an item
 #define ST_ACTIVE       0x04    // station is active (deleted objects are
@@ -486,10 +522,12 @@ extern CADRow *CAD_list_head;
 #define ST_MOVING       0x08    // station is moving
 #define ST_DIRECT       0x10    // heard direct (not via digis)
 #define ST_VIATNC       0x20    // station heard via TNC
-#define ST_3RD_PT       0x40    // third party traffic (not used yet)
+#define ST_3RD_PT       0x40    // third party traffic
 #define ST_MSGCAP       0x80    // message capable (not used yet)
 #define ST_STATUS       0x100   // got real status message
 #define ST_INVIEW       0x200   // station is in current screen view
+#define ST_MYSTATION    0x400   // station is owned by my call-SSID
+#define ST_MYOBJITEM    0x800   // object/item owned by me
 
 
 #ifdef DATA_DEBUG
@@ -503,9 +541,13 @@ extern void copy_data(DataRow *to, DataRow *from);
 
 
 extern void db_init(void);
+extern void export_trail_as_kml(DataRow *p_station);   // export trail of one or all stations to kml file
 
 //
 extern int is_my_call(char *call, int exact);
+extern int is_my_station(DataRow *p_station);
+extern int is_my_object_item(DataRow *p_station);
+
 void mscan_file(char msg_type, void (*function)(Message *fill));
 extern void msg_record_ack(char *to_call_sign, char *my_call, char *seq,
                            int timeout, int cancel);
@@ -519,10 +561,9 @@ extern void mem_display(void);
 extern long sort_input_database(char *filename, char *fill, int size);
 extern void sort_display_file(char *filename, int size);
 extern void clear_sort_file(char *filename);
-extern int  packet_data_display;
 extern void display_packet_data(void);
 extern int  redraw_on_new_packet_data;
-extern int decode_ax25_header(unsigned char *incoming_data, int length);
+extern int decode_ax25_header(unsigned char *data_string, int *length);
 extern int decode_ax25_line(char *line, char from, int port, int dbadd);
 
 // utilities
@@ -538,7 +579,7 @@ extern void update_messages(int force);
 extern void mdelete_messages_from(char *from);
 extern void mdelete_messages_to(char *to);
 extern void init_message_data(void);
-extern void check_message_remove(void);
+extern void check_message_remove(time_t curr_sec);
 extern int  new_message_data;
 extern time_t msg_data_add(char *call_sign, char *from_call, char *data,
                            char *seq, char type, char from, long *record_out);
@@ -551,11 +592,12 @@ extern DataRow *n_first;  // pointer to first element in name ordered station
                           // list
 extern DataRow *n_last;   // pointer to last element in name ordered station
                           // list
-extern DataRow *t_first;  // pointer to first element in time ordered station
+extern DataRow *t_oldest; // pointer to first element in time ordered station
                           // list
-extern DataRow *t_last;   // pointer to last element in time ordered station
+extern DataRow *t_newest; // pointer to last element in time ordered station
                           // list
 extern void init_station_data(void);
+extern void Station_data(Widget w, XtPointer clientData, XtPointer calldata);
 extern int station_data_auto_update;
 extern int  next_station_name(DataRow **p_curr);
 extern int  prev_station_name(DataRow **p_curr);
@@ -563,7 +605,7 @@ extern int  next_station_time(DataRow **p_curr);
 extern int  prev_station_time(DataRow **p_curr);
 extern int  search_station_name(DataRow **p_name, char *call, int exact);
 extern int  search_station_time(DataRow **p_time, time_t heard, int serial);
-extern void check_station_remove(void);
+extern void check_station_remove(time_t curr_sec);
 extern void delete_all_stations(void);
 extern void station_del(char *callsign);
 extern void my_station_add(char *my_call_sign, char my_group, char my_symbol,
@@ -577,11 +619,6 @@ extern int  locate_station(Widget w, char *call, int follow_case,
 extern void update_station_info(Widget w);
 
 // objects/items
-extern void compute_DR_position(long x_long, long y_lat, double range, double course, long *x_long2, long *y_lat2);
-extern void compute_current_DR_position(DataRow *p_station, long *x_long, long *y_lat);
-extern void check_and_transmit_objects_items(time_t time);
-extern int Create_object_item_tx_string(DataRow *p_station, char *line, 
-                                        int line_length);
 extern time_t last_object_check;
 
 // trails
@@ -613,10 +650,20 @@ typedef struct {
     int total;
 } aloha_stats;
 
-double calc_aloha_distance(); //meat
-void calc_aloha(); // periodic function
+double calc_aloha_distance(void); //meat
+void calc_aloha(int curr_sec); // periodic function
 void Show_Aloha_Stats(Widget w, XtPointer clientData, 
                       XtPointer callData); // popup window
 
 int comp_by_dist(const void *,const void *);// used only for qsort
+DataRow * sanity_check_time_list(time_t); // used only for debugging
+void dump_time_sorted_list(void);
+
+extern int store_trail_point(DataRow *p_station, long lon, long lat, time_t sec, char *alt, char *speed, char *course, short stn_flag);
+
+#ifdef HAVE_DB
+extern int add_simple_station(DataRow *p_new_station,char *station, char *origin, char *symbol, char *overlay, char *aprs_type, char *latitude, char *longitude, char *record_type, char *node_path, char *transmit_time, char *timeformat);
+
+#endif /* HAVE_DB */
+
 #endif /* XASTIR_DATABASE_H */
