@@ -1,8 +1,8 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: dbfawk.c,v 1.31 2006/04/24 19:21:41 we7u Exp $
+ * $Id: dbfawk.c,v 1.38 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
- * Copyright (C) 2003-2006  The Xastir Group
+ * Copyright (C) 2003-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -144,7 +144,8 @@ dbfawk_field_info *dbfawk_field_list(DBFHandle dbf, char *dbffields) {
     d = fi->name;
     while (*p && *p != ':') *d++ = *p++;
     if (*p == ':')
-      *d = *p++ = '\0';
+      *p++ = '\0';
+    *d='\0';
     fi->num = DBFGetFieldIndex(dbf, fi->name);
     fi->type = DBFGetFieldInfo(dbf, fi->num, junk, &w, &prec);
     sp = p;
@@ -231,6 +232,7 @@ dbfawk_sig_info *dbfawk_load_sigs(const char *dir, /* directory path */
             if (symtbl)
                 awk_free_symtab(symtbl);
             fprintf(stderr,"failed to malloc in dbfawk.c!\n");
+	    closedir(d);
             return NULL;
         }
  
@@ -242,6 +244,9 @@ dbfawk_sig_info *dbfawk_load_sigs(const char *dir, /* directory path */
                 if (!i) {
                     fprintf(stderr,"failed to malloc in dbfawk.c!\n");
                     free(path);
+                    if (symtbl)
+                      awk_free_symtab(symtbl);
+		    closedir(d);
                     return NULL;
                 }
             } else {
@@ -250,7 +255,10 @@ dbfawk_sig_info *dbfawk_load_sigs(const char *dir, /* directory path */
                 if (!i->next) {
                     fprintf(stderr,"failed to malloc in dbfawk.c!\n");
                     free(path);
-                    return NULL;
+                    if (symtbl)
+                      awk_free_symtab(symtbl);
+		    closedir(d);
+                    return head; // Return what we were able to gather.
                 }
 
                 i = i->next;
