@@ -1,9 +1,9 @@
 /*
- * $Id: fcc_data.c,v 1.13 2005/02/04 19:36:06 we7u Exp $
+ * $Id: fcc_data.c,v 1.25 2012/09/23 16:19:22 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -48,17 +51,21 @@
 #include "xa_config.h"
 #include "main.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
 
 
 
 
 char *call_only(char *callsign) {
-    int i;
+    int i, len;
 
-    for (i=0; i<(int)strlen(callsign); i++) {
+    len = strlen(callsign);
+    for (i = 0; i < len; i++) {
          if (!isalnum((int)callsign[i])) {
             callsign[i]='\0';
-            i=strlen(callsign)+1;
+            i=len;
         }
     }
     return(callsign);
@@ -68,9 +75,9 @@ char *call_only(char *callsign) {
 
 
 
-/* ====================================================================    */
-/*    build a new (or newer if I check the file date) index file    */
-/*    check for current ic index file                    */
+/* ====================================================================  */
+/*    build a new (or newer if I check the file date) index file	 */
+/*    check for current ic index file					 */
 /*    FG: added a date check in case the FCC file has been updated.    */
 /*      appl.dat must have a time stamp newer than the index file time  */
 /*      stamp. Use the touch command on the appl.dat file to make the   */
@@ -87,6 +94,7 @@ int build_fcc_index(int type){
     char fccdata[FCC_DATA_LEN+8];
     char database_name[100];
     int found,i,num;
+    char appl_file_path[MAX_VALUE];
 
     if (type==1)
         xastir_snprintf(database_name, sizeof(database_name), "fcc/appl.dat");
@@ -96,12 +104,16 @@ int build_fcc_index(int type){
     /* ====================================================================    */
     /*    If the index file is there, exit                */
     /*                                    */
-    if (filethere(get_user_base_dir("data/appl.ndx"))) {
+    get_user_base_dir("data/appl.ndx", appl_file_path, 
+                      sizeof(appl_file_path));
+    if (filethere(appl_file_path)) {
         /* if file is there make sure the index date is newer */
-        if (file_time(get_data_base_dir(database_name))<=file_time(get_user_base_dir("data/appl.ndx")))
+        if (file_time(get_data_base_dir(database_name))<=file_time(appl_file_path))
             return(1);
         else {
-            statusline("FCC index old, rebuilding",1);
+            // FCC index old, rebuilding
+            statusline(langcode("STIFCC0100"),1);
+
             fprintf(stderr,"FCC index is old.  Rebuilding index.\n");
 //            XmTextFieldSetString(text,"FCC index old, rebuilding");
 //            XtManageChild(text);
@@ -118,9 +130,9 @@ int build_fcc_index(int type){
         return(0);
     }
 
-    fndx=fopen(get_user_base_dir("data/appl.ndx"),"w");
+    fndx=fopen(appl_file_path,"w");
     if (fndx==NULL){
-        fprintf(stderr,"Build:Could not open/create FCC data base index: %s\n", get_user_base_dir("data/appl.ndx") );
+        fprintf(stderr,"Build:Could not open/create FCC data base index: %s\n", appl_file_path );
         (void)fclose(fdb);
         return(0);
     }
@@ -235,6 +247,7 @@ int search_fcc_data_appl(char *callsign, FccAppl *data) {
     long call_offset = 0;
     char char_offset[16];
     char index[32];
+    char appl_file_path[MAX_VALUE];
 
     data->id_file_num[0] = '\0';
     data->type_purpose[0] = '\0';
@@ -260,7 +273,7 @@ int search_fcc_data_appl(char *callsign, FccAppl *data) {
 // calltemp doesn't appear to get used anywhere...
 
     /* add end of field data */
-    strncat(temp, "|", sizeof(temp) - strlen(temp));
+    strncat(temp, "|", sizeof(temp) - 1 - strlen(temp));
     len=(int)strlen(temp);
     found=0;
     line_pos=0;
@@ -277,7 +290,8 @@ int search_fcc_data_appl(char *callsign, FccAppl *data) {
     // This gives us a jumping-off point to start looking in the right
     // neighborhood for the callsign of interest.
     //
-    fndx=fopen(get_user_base_dir("data/appl.ndx"),"r");
+    get_user_base_dir("data/appl.ndx", appl_file_path, sizeof(appl_file_path));
+    fndx=fopen(appl_file_path,"r");
     if (fndx!=NULL){
         (void)fgets(index,(int)sizeof(index),fndx);
         xastir_snprintf(char_offset,sizeof(char_offset),"%s",&index[6]);
@@ -290,7 +304,7 @@ int search_fcc_data_appl(char *callsign, FccAppl *data) {
             (void)fgets(index,(int)sizeof(index),fndx);
         }
     } else {
-        fprintf(stderr,"Search:Could not open FCC data base index: %s\n", get_user_base_dir("data/appl.ndx") );
+        fprintf(stderr,"Search:Could not open FCC data base index: %s\n", appl_file_path );
         return (0);
     }
     call_offset = atol(char_offset);
@@ -465,7 +479,11 @@ int search_fcc_data_appl(char *callsign, FccAppl *data) {
                             // location for the callsign.  Return if so.
                             if ( (temp[0] < line[pos]) ||
                                     ( (temp[0] == line[pos]) && (temp[1] < line[pos+1]) ) ) {
-                                popup_message_always("Callsign Search", "Callsign Not Found!");
+
+                                // "Callsign Search", "Callsign Not Found!"
+                                popup_message_always(langcode("STIFCC0101"),
+                                    langcode("STIFCC0102") );
+
                                 //fprintf(stderr,"%c%c\t%c%c\n",temp[0],temp[1],line[pos],line[pos+1]);
                                 (void)fclose(f);
                                 return(0);
