@@ -1,9 +1,9 @@
 /*
- * $Id: gps.c,v 1.48 2005/06/01 19:24:44 we7u Exp $
+ * $Id: gps.c,v 1.57 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -56,6 +59,10 @@
 #include "lang.h"
 #include "util.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 char gps_gprmc[MAX_GPS_STRING+1];
 char gps_gpgga[MAX_GPS_STRING+1];
@@ -107,7 +114,7 @@ int decode_gps_rmc( char *data,
 
     char *temp_ptr;
     char *temp_ptr2;
-    char temp_data[MAX_TNC_LINE_SIZE+1];    // Big in case we get concatenated packets (it happens!)
+    char temp_data[MAX_LINE_SIZE+1];    // Big in case we get concatenated packets (it happens!)
     char sampletime[7]; // We ignore fractional seconds
     char long_pos_x[11];
     char long_ew;
@@ -369,7 +376,7 @@ int decode_gps_gga( char *data,
 
     char *temp_ptr;
     char *temp_ptr2;
-    char temp_data[MAX_TNC_LINE_SIZE+1];    // Big in case we get concatenated packets (it happens!)
+    char temp_data[MAX_LINE_SIZE+1];    // Big in case we get concatenated packets (it happens!)
     char long_pos_x[11];
     char long_ew;
     char lat_pos_y[10];
