@@ -1,9 +1,9 @@
 /*
- * $Id: igate.c,v 1.47 2005/03/16 21:36:11 we7u Exp $
+ * $Id: igate.c,v 1.66 2012/09/23 16:19:22 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,11 +22,13 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <termios.h>
-#include <pthread.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <ctype.h>
@@ -45,6 +47,10 @@
 #include "xa_config.h"
 #include "util.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 time_t last_nws_stations_file_time = 0;
 int NWS_stations = 0;
@@ -62,7 +68,7 @@ int check_NWS_stations(char *call);
 // pointer set to NULL.
 //
 typedef struct _DupeRecord{
-    char    data[MAX_TNC_LINE_SIZE+15]; // Packet data
+    char    data[MAX_LINE_SIZE+15]; // Packet data
     time_t  time;                       // The time the record was inserted
     struct  _DupeRecord *next;          // pointer to next record in list
 } DupeRecord;
@@ -139,8 +145,8 @@ int not_a_dupe(int queue_type, int port, char *line, int insert_mode) {
     int insert_new;
     time_t time_cutoff;
     int found_dupe = 0;
-    char match_line[MAX_TNC_LINE_SIZE*2];
-    char line2[MAX_TNC_LINE_SIZE+1];
+    char match_line[MAX_LINE_SIZE*2];
+    char line2[MAX_LINE_SIZE+1];
     char *c0, *c1, *c2;
 
 
@@ -155,7 +161,7 @@ int not_a_dupe(int queue_type, int port, char *line, int insert_mode) {
     // Fill the destination string with zeroes.  This is a nice
     // segfault-prevention technique.  Whatever strings we throw in here
     // will be automatically terminated.
-    memset(match_line, 0, MAX_TNC_LINE_SIZE*2);
+    memset(match_line, 0, MAX_LINE_SIZE*2);
 
 
     switch (queue_type) {
@@ -190,6 +196,7 @@ int not_a_dupe(int queue_type, int port, char *line, int insert_mode) {
             // Changes needed before parsing code:  Get rid of first
             // part of packet up to the '}' symbol.  After this the
             // generic parsing code will work.
+// Note that the REPLY-ACK algorithm also uses the '}' symbol.
 
             c0 = strstr(line, ":}"); // Find start of 3rd party packet
             if (c0 == NULL) {   // Not 3rd party packet
@@ -201,6 +208,7 @@ int not_a_dupe(int queue_type, int port, char *line, int insert_mode) {
             // Copy original packet into line2 for later parsing.  We
             // want to keep the '}' character because our own
             // transmissions out RF have that character as well.
+// Note that the REPLY-ACK algorithm also uses the '}' symbol.
             if (debug_level & 1024)
                 fprintf(stderr,"3rd party HeardQ: %s\n",line);
 
@@ -255,7 +263,7 @@ int not_a_dupe(int queue_type, int port, char *line, int insert_mode) {
 
         strncat(match_line,                         // Copy info portion
             c2+1,
-            sizeof(match_line) - strlen(match_line));
+            sizeof(match_line) - 1 - strlen(match_line));
     }
     else {  // At least one separator was not found, copy entire string
         xastir_snprintf(match_line,
@@ -482,6 +490,7 @@ void output_igate_net(char *line, int port, int third_party) {
     char *message;
     int len,i,x,first;
     int igate_options;
+    char log_file_path[MAX_VALUE];
 
     call_sign = NULL;
     path      = NULL;
@@ -520,6 +529,7 @@ void output_igate_net(char *line, int port, int third_party) {
     if (path == NULL)
         return;
 
+    get_user_base_dir(LOGFILE_IGATE,log_file_path, sizeof(log_file_path));
     // Check for "TCPIP" or "TCPXX" in the path.  If found, don't
     // gate this into the internet again, it's already been gated to
     // RF, which means it's already been on the 'net.  No looping
@@ -545,14 +555,14 @@ void output_igate_net(char *line, int port, int third_party) {
                     "IGATE RF->NET(%c):%s\n",
                     third_party ? 'T':'N',
                     line);
-                log_data(LOGFILE_IGATE,temp);
+                log_data( log_file_path, temp );
 
                 xastir_snprintf(temp,
                     sizeof(temp),
                     "REJECT: Packet was gated before or shouldn't be gated!\n");
-                log_data(LOGFILE_IGATE,temp);
+                log_data( log_file_path, temp );
 
-                fprintf(stderr,temp);
+                fprintf(stderr, "%s", temp);
         }
         return;
     }
@@ -564,6 +574,7 @@ void output_igate_net(char *line, int port, int third_party) {
 
     // Check for third party messages.  We don't want to gate these
     // back onto the internet feeds
+// Note that the REPLY-ACK algorithm also uses the '}' symbol.
     if (message[0] == '}') {
 
         if (log_igate && (debug_level & 1024) ) {
@@ -573,21 +584,27 @@ void output_igate_net(char *line, int port, int third_party) {
                 "IGATE RF->NET(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: Third party traffic!\n");
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
-            fprintf(stderr,temp);
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
 
-    // Check for generic queries, which have a '?' mark as the first
-    // character in the message field.  We don't wish to gate these.
-    if (message[0] == '?') {
+    // Check for "general" queries.  We don't wish to gate these in
+    // either direction.  There are exactly three general query
+    // types defined in the spec.
+    //
+    if ( (   strstr(message,"?APRS?" ) != NULL)
+         || (strstr(message,"?IGATE?") != NULL)
+         || (strstr(message,"?WX?"   ) != NULL) ) {
+
+        // We found a general query, don't gate it.
 
         if (log_igate && (debug_level & 1024) ) {
 
@@ -596,14 +613,14 @@ void output_igate_net(char *line, int port, int third_party) {
                 "IGATE RF->NET(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
-                "REJECT: Generic Query!\n");
-            log_data(LOGFILE_IGATE,temp);
+                "REJECT: General Query!\n");
+            log_data( log_file_path, temp );
 
-            fprintf(stderr,temp);
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
@@ -628,14 +645,14 @@ void output_igate_net(char *line, int port, int third_party) {
                 "IGATE RF->NET(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: From my call!\n");
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
-            fprintf(stderr,temp);
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
@@ -681,15 +698,15 @@ end_critical_section(&devices_lock, "igate.c:output_igate_net" );
                 "IGATE RF->NET(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: No RF->NET from input port [%d]!\n",
                 port);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
-            fprintf(stderr,temp);
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
@@ -724,7 +741,7 @@ end_critical_section(&devices_lock, "igate.c:output_igate_net" );
                         "IGATE RF->NET(%c):%s\n",
                         third_party ? 'T':'N',
                         line);
-                    log_data(LOGFILE_IGATE,temp);
+                    log_data( log_file_path, temp );
 
                     first = 0;
                 }
@@ -738,7 +755,7 @@ end_critical_section(&devices_lock, "igate.c:output_igate_net" );
 
                 // log output
                 if (log_igate)
-                    log_data(LOGFILE_IGATE,temp);
+                    log_data( log_file_path, temp );
 
                 if (debug_level & 1024)
                     fprintf(stderr,"%s\n",temp);
@@ -770,8 +787,9 @@ void output_igate_rf(char *from, char *call, char *path, char *line,
     char temp[MAX_LINE_SIZE+20];
     int x;
     int first = 1;
-    char *ii;
-    char *jj;
+    int found_in_nws_file = 0;
+    char log_file_path[MAX_VALUE];
+    char nws_file_path[MAX_VALUE];
 
 
     if ( (from == NULL) || (call == NULL) || (path == NULL) || (line == NULL) )
@@ -784,28 +802,8 @@ void output_igate_rf(char *from, char *call, char *path, char *line,
     if (operate_as_an_igate <= 1)
         return;
 
-    // Check for TCPXX in string.  If found, we have an
-    // unregistered net user.
-    // I removed the trailing asterisk -we7u
-    if (strstr(path,"TCPXX") != NULL) {
-        // "TCPXX" was found in the header.  We have an
-        // unregistered user.
-        if (log_igate && (debug_level & 1024) ) {
-            xastir_snprintf(temp,
-                sizeof(temp),
-                "IGATE NET->RF(%c):%s\n",
-                third_party ? 'T':'N',
-                line);
-            log_data(LOGFILE_IGATE,temp);
 
-            xastir_snprintf(temp,
-                sizeof(temp),
-                "REJECT: Unregistered net user!\n");
-            log_data(LOGFILE_IGATE,temp);
-            fprintf(stderr,temp);
-        }
-        return;
-    }
+    get_user_base_dir(LOGFILE_IGATE,log_file_path, sizeof(log_file_path));
 
     // Don't gate anything with NOGATE in it, in either direction.
     // Same for OpenTrac packets.
@@ -819,57 +817,135 @@ void output_igate_rf(char *from, char *call, char *path, char *line,
                 "IGATE NET->RF(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: NOGATE found in path or shouldn't be gated!\n");
-            log_data(LOGFILE_IGATE,temp);
-            fprintf(stderr,temp);
+            log_data( log_file_path, temp );
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
 
-    // Don't gate generic queries in any direction.  These packets
-    // have a '?' character after the initial ':' character.
+    // Don't gate "general" queries in any direction.  There are
+    // exactly three general query types defined in the spec.
     //
-    ii = strstr(path,":");  // Find the first ':' character
-    jj = strstr(path,":?"); // Find ":?" combination
+    if (   (strstr(line,"?APRS?" ) != NULL)
+        || (strstr(line,"?IGATE?") != NULL)
+        || (strstr(line,"?WX?"   ) != NULL) ) {
+
+        // We found a general query, don't gate it.
 
-    if ( (jj != NULL) && (ii == jj) ) {
-        // We found a generic query.  Don't gate it.
         if (log_igate && (debug_level & 1024) ) {
             xastir_snprintf(temp,
                 sizeof(temp),
                 "IGATE NET->RF(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
-                "REJECT: Generic query!\n");
-            log_data(LOGFILE_IGATE,temp);
-            fprintf(stderr,temp);
+                "REJECT: General Query!\n");
+            log_data( log_file_path, temp );
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
 
 
      // check to see if the nws-stations file is newer than last read
-    if (last_nws_stations_file_time < file_time(get_user_base_dir("data/nws-stations.txt"))) {
-        last_nws_stations_file_time = file_time(get_user_base_dir("data/nws-stations.txt"));
-        load_NWS_stations(get_user_base_dir("data/nws-stations.txt"));
+    get_user_base_dir("data/nws-stations.txt",nws_file_path, 
+                      sizeof(nws_file_path));
+    if (last_nws_stations_file_time < file_time(nws_file_path)) {
+        last_nws_stations_file_time = file_time(nws_file_path);
+        load_NWS_stations(nws_file_path);
         //fprintf(stderr,"NWS Station file time is old\n");
     }
 
 
+    // Check whether gating of packets from this station/object/item
+    // has been specifically authorized via the nws-stations.txt
+    // mechanism.
+    //
+    if (object_name) {  // It's an object or item name
+
+        if ( check_NWS_stations( object_name ) || group_active(object_name)) {
+
+            found_in_nws_file++; // Object/Item is in nws-stations.txt
+        }
+    }
+    else {              // It's a station callsign
+
+        if ( check_NWS_stations( from ) || group_active(from)) {
+
+            found_in_nws_file++; // Source callsign is in nws-stations.txt
+        }
+    }
+// The above is really the same as the following code, but less
+// confusing:
+//    if (check_NWS_stations( (object_name) ? object_name : from ) ) {
+//        found_in_nws_file++;
+//    }      
+
+ 
+    // Check for TCPXX in string only if station wasn't found in the
+    // nws-stations.txt file.  If TCPXX found, we have an
+    // unregistered net user and the packet shouldn't normally head
+    // to RF.
+    //
+    // I removed the trailing asterisk -we7u
+    //
+    // Note that we CAN now gate stations to RF that have TCPXX in
+    // the string if they are authorized via the nws-stations.txt
+    // mechanism.
+    //
+    if (!found_in_nws_file) { // Skip this check if they're always authorized via the file
+
+        if (strstr(path,"TCPXX") != NULL) {
+
+            // "TCPXX" was found in the header.  We have an
+            // unregistered user.
+
+            if (log_igate && (debug_level & 1024) ) {
+                xastir_snprintf(temp,
+                    sizeof(temp),
+                    "IGATE NET->RF(%c):%s\n",
+                    third_party ? 'T':'N',
+                    line);
+                log_data( log_file_path, temp );
+    
+                xastir_snprintf(temp,
+                    sizeof(temp),
+                    "REJECT: Unregistered net user!\n");
+                log_data( log_file_path, temp );
+                fprintf(stderr, "%s", temp);
+            }
+            return;
+        }
+    }
+
+
+
+// If we made it to this point, the packet is from an authorized net
+// user (no TCPXX found in the path), or the callsign has been
+// authorized via the nws-stations.txt file (whether or not TCPXX
+// was found in the path).
+//
+// Found in file: Gate always
+//
+// Not found:     Gate if TCPXX not in path -AND- if destination
+//                station was heard in last hour on RF -AND- if
+//                source station was NOT heard in last hour on RF.
+
+
+
     // Check whether the source and destination calls have been
     // heard on local RF.
-    if (  (   (!object_name && !check_NWS_stations(from)) // Source call not in nws-stations.txt
-            || (object_name && !check_NWS_stations(object_name))) // Object/item not in nws-stations.txt
-        && (!heard_via_tnc_in_past_hour(call)==1        // Haven't heard destination call in previous hour
-            || heard_via_tnc_in_past_hour(from))) {  // Have heard source call in previous hour
+    if ( !found_in_nws_file // Skip this check if they're always authorized via the file
+         && ( !heard_via_tnc_in_past_hour(call)==1    // Haven't heard destination call in previous hour
+            || heard_via_tnc_in_past_hour(from)) ) {  // Have heard source call in previous hour
 
         if (log_igate && (debug_level & 1024) ) {
             xastir_snprintf(temp,
@@ -877,7 +953,7 @@ void output_igate_rf(char *from, char *call, char *path, char *line,
                 "IGATE NET->RF(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
         //  heard(call),  heard(from) : RF-to-RF talk
         // !heard(call),  heard(from) : Destination not heard on TNC
@@ -892,8 +968,8 @@ void output_igate_rf(char *from, char *call, char *path, char *line,
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: RF->RF talk!\n");
-            log_data(LOGFILE_IGATE,temp);
-            fprintf(stderr,temp);
+            log_data( log_file_path, temp );
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
@@ -945,7 +1021,7 @@ begin_critical_section(&devices_lock, "igate.c:output_igate_rf" );
                                 "IGATE NET->RF(%c):%s\n",
                                 third_party ? 'T':'N',
                                 line);
-                            log_data(LOGFILE_IGATE,temp);
+                            log_data( log_file_path, temp );
                             first = 0;
                         }
 
@@ -956,10 +1032,10 @@ begin_critical_section(&devices_lock, "igate.c:output_igate_rf" );
 
                         // log output
                         if (log_igate)
-                            log_data(LOGFILE_IGATE,temp);
+                            log_data( log_file_path, temp );
 
                         if (debug_level & 1024)
-                            fprintf(stderr,temp);
+                            fprintf(stderr, "%s", temp);
 
                         // ok write this data out to the RF port
 
@@ -982,14 +1058,14 @@ begin_critical_section(&devices_lock, "igate.c:output_igate_rf" );
                                 "IGATE NET->RF(%c):%s\n",
                                 third_party ? 'T':'N',
                                 line);
-                            log_data(LOGFILE_IGATE,temp);
+                            log_data( log_file_path, temp );
 
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "REJECT: NET->RF on port [%d]!\n",
                                 x);
-                            log_data(LOGFILE_IGATE,temp);
-                            fprintf(stderr,temp);
+                            log_data( log_file_path, temp );
+                            fprintf(stderr, "%s", temp);
                         }
                     }
 
@@ -1129,7 +1205,7 @@ int check_NWS_stations(char *call) {
             if (strncasecmp(call, NWS_station_data[i].call, length)==0) {
 
                 ok=1; // match found 
-                if (debug_level && 1024) {
+                if (debug_level & 1024) {
                     fprintf(stderr,"NWS-MATCH:(%s) (%s)\n",NWS_station_data[i].call,call);
                 }
             }
@@ -1156,6 +1232,8 @@ void output_nws_igate_rf(char *from, char *path, char *line, int port, int third
     char temp[MAX_LINE_SIZE+20];
     int x;
     int first = 1;
+    char log_file_path[MAX_VALUE];
+    char nws_file_path[MAX_VALUE];
 
 
     if ( (from == NULL) || (path == NULL) || (line == NULL) )
@@ -1168,6 +1246,10 @@ void output_nws_igate_rf(char *from, char *path, char *line, int port, int third
     if (operate_as_an_igate <= 1)
         return;
 
+    get_user_base_dir(LOGFILE_IGATE,log_file_path, sizeof(log_file_path));
+    get_user_base_dir("data/nws-stations.txt",nws_file_path, 
+                      sizeof(nws_file_path));
+
     // Check for TCPXX in string!  If found, we have an
     // unregistered net user.
     // I removed the trailing asterisk --we7u
@@ -1180,13 +1262,13 @@ void output_nws_igate_rf(char *from, char *path, char *line, int port, int third
                 "NWS IGATE NET->RF(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: Unregistered net user!\n");
-            log_data(LOGFILE_IGATE,temp);
-            fprintf(stderr,temp);
+            log_data( log_file_path, temp );
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
@@ -1202,45 +1284,45 @@ void output_nws_igate_rf(char *from, char *path, char *line, int port, int third
                 "NWS IGATE NET->RF(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: NOGATE found in path!\n");
-            log_data(LOGFILE_IGATE,temp);
-            fprintf(stderr,temp);
+            log_data( log_file_path, temp );
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
  
     // see if we can gate NWS messages
-    if (!filethere(get_user_base_dir("data/nws-stations.txt"))) {
+    if (!filethere(nws_file_path)) {
         if (log_igate && (debug_level & 1024) ) {
             xastir_snprintf(temp,
                 sizeof(temp),
                 "NWS IGATE NET->RF(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: No nws-stations.txt file!\n");
-            log_data(LOGFILE_IGATE,temp);
-            fprintf(stderr,temp);
+            log_data( log_file_path, temp );
+            fprintf(stderr, "%s", temp);
         }
         return;
     }
 
     // check to see if the nws-stations file is newer than last read
-    if (last_nws_stations_file_time < file_time(get_user_base_dir("data/nws-stations.txt"))) {
-        last_nws_stations_file_time = file_time(get_user_base_dir("data/nws-stations.txt"));
-        load_NWS_stations(get_user_base_dir("data/nws-stations.txt"));
+    if (last_nws_stations_file_time < file_time(nws_file_path)) {
+        last_nws_stations_file_time = file_time(nws_file_path);
+        load_NWS_stations(nws_file_path);
         //fprintf(stderr,"NWS Station file time is old\n");
     }
 
     // Look for NWS station in file data
-    if (!check_NWS_stations(from)){ // Couldn't find the station
+    if (!check_NWS_stations(from) || !group_active(from)){ // Couldn't find the station
 
         if (log_igate && (debug_level & 1024) ) {
             xastir_snprintf(temp,
@@ -1248,13 +1330,13 @@ void output_nws_igate_rf(char *from, char *path, char *line, int port, int third
                 "NWS IGATE NET->RF(%c):%s\n",
                 third_party ? 'T':'N',
                 line);
-            log_data(LOGFILE_IGATE,temp);
+            log_data( log_file_path, temp );
 
             xastir_snprintf(temp,
                 sizeof(temp),
                 "REJECT: No matching station in nws-stations.txt file!\n");
-            log_data(LOGFILE_IGATE,temp);
-            fprintf(stderr,temp);
+            log_data( log_file_path, temp );
+            fprintf(stderr, "%s", temp);
         }
         return; // Match for station not found in file
     }
@@ -1301,7 +1383,7 @@ begin_critical_section(&devices_lock, "igate.c:output_nws_igate_rf" );
                                 "NWS IGATE NET->RF(%c):%s\n",
                                 third_party ? 'T':'N',
                                 line);
-                            log_data(LOGFILE_IGATE,temp);
+                            log_data( log_file_path, temp );
                             first = 0;
                         }
 
@@ -1312,10 +1394,10 @@ begin_critical_section(&devices_lock, "igate.c:output_nws_igate_rf" );
 
                         // log output
                         if (log_igate)
-                            log_data(LOGFILE_IGATE,temp);
+                            log_data( log_file_path, temp );
 
                         if (debug_level & 1024)
-                            fprintf(stderr,temp);
+                            fprintf(stderr, "%s", temp);
 
                         // ok write this data out to the RF port
 
@@ -1336,14 +1418,14 @@ begin_critical_section(&devices_lock, "igate.c:output_nws_igate_rf" );
                                 "NWS IGATE NET->RF(%c):%s\n",
                                 third_party ? 'T':'N',
                                 line);
-                            log_data(LOGFILE_IGATE,temp);
+                            log_data( log_file_path, temp );
 
                             xastir_snprintf(temp,
                                 sizeof(temp),
                                 "REJECT: NET->RF on port [%d]!\n",
                                 x);
-                            log_data(LOGFILE_IGATE,temp);
-                            fprintf(stderr,temp);
+                            log_data( log_file_path, temp );
+                            fprintf(stderr, "%s", temp);
                         }
                     }
 
