@@ -1,9 +1,9 @@
 /*
- * $Id: locate_gui.c,v 1.18 2005/07/31 06:19:12 we7u Exp $
+ * $Id: locate_gui.c,v 1.32 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -37,6 +40,10 @@
 #include "lang.h"
 #include "maps.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+extern XmFontList fontlist1;    // Menu/System fontlist
 
 Widget locate_station_dialog = (Widget)NULL;
 Widget locate_station_data = (Widget)NULL;
@@ -138,6 +145,7 @@ void fcc_rac_lookup(Widget w, XtPointer clientData, XtPointer callData) {
     XtFree(temp_ptr);
 
     (void)remove_trailing_spaces(station_call);
+    (void)remove_trailing_dash_zero(station_call);
 
     to_upper(station_call);
 
@@ -190,26 +198,26 @@ void fcc_rac_lookup(Widget w, XtPointer clientData, XtPointer callData) {
                     if (my_rac_data.qual_a[0] == 'A')
                         strncat(temp,
                             langcode("STIFCC0008"),
-                            sizeof(temp) - strlen(temp));
+                            sizeof(temp) - 1 - strlen(temp));
 
                     if (my_rac_data.qual_d[0] == 'D')
                         strncat(temp,
                             langcode("STIFCC0009"),
-                            sizeof(temp) - strlen(temp));
+                            sizeof(temp) - 1 - strlen(temp));
 
                     if (my_rac_data.qual_b[0] == 'B' && my_rac_data.qual_c[0] != 'C')
                         strncat(temp,
                             langcode("STIFCC0010"),
-                            sizeof(temp) - strlen(temp));
+                            sizeof(temp) - 1 - strlen(temp));
 
                     if (my_rac_data.qual_c[0] == 'C')
                         strncat(temp,
                             langcode("STIFCC0011"),
-                            sizeof(temp) - strlen(temp));
+                            sizeof(temp) - 1 - strlen(temp));
 
                     strncat(temp,
                         "\n",
-                        sizeof(temp) - strlen(temp));
+                        sizeof(temp) - 1 - strlen(temp));
 
                     if (strlen(my_rac_data.club_name) > 1) {
                         xastir_snprintf(temp2,
@@ -222,7 +230,7 @@ void fcc_rac_lookup(Widget w, XtPointer clientData, XtPointer callData) {
                             my_rac_data.club_postal_code);
                         strncat(temp,
                             temp2,
-                            sizeof(temp) - strlen(temp));
+                            sizeof(temp) - 1 - strlen(temp));
                     }
 
 
@@ -240,7 +248,10 @@ void fcc_rac_lookup(Widget w, XtPointer clientData, XtPointer callData) {
             popup_message_always(langcode("POPEM00001"),temp2);
             break;
     }
-    Locate_station_destroy_shell(w, clientData, callData);
+
+    // Don't enable this as then we can't click on the Locate button
+    // later.
+    //Locate_station_destroy_shell(w, clientData, callData);
 }
 
 
@@ -264,13 +275,19 @@ void Locate_station_now(Widget w, XtPointer clientData, XtPointer callData) {
     XtFree(temp_ptr);
 
     (void)remove_trailing_spaces(locate_station_call);
+    (void)remove_trailing_dash_zero(locate_station_call);
+
     /*fprintf(stderr,"looking for %s\n",locate_station_call);*/
     if (locate_station(da, locate_station_call, (int)XmToggleButtonGetState(locate_case_data),
                     (int)XmToggleButtonGetState(locate_match_data),1) ==0) {
         xastir_snprintf(temp2, sizeof(temp2), langcode("POPEM00002"), locate_station_call);
         popup_message_always(langcode("POPEM00001"),temp2);
     }
-    Locate_station_destroy_shell(w, clientData, callData);
+
+    // Don't enable this as then we can't click on the FCC/RAC
+    // button later, and we'll lose the callsign info if we want to
+    // see it again.
+    //Locate_station_destroy_shell(w, clientData, callData);
 }
 
 
@@ -294,16 +311,18 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
         // Check whether it is an emergency locate function
         if (emergency_flag == 1) {
                 locate_station_dialog = XtVaCreatePopupShell(langcode("WPUPLSP006"),
-                xmDialogShellWidgetClass,Global.top,
-                XmNdeleteResponse,XmDESTROY,
+                xmDialogShellWidgetClass, appshell,
+                XmNdeleteResponse, XmDESTROY,
                 XmNdefaultPosition, FALSE,
+                XmNfontList, fontlist1,
                 NULL);
         }
         else {  // Non-emergency locate
             locate_station_dialog = XtVaCreatePopupShell(langcode("WPUPLSP001"),
-                xmDialogShellWidgetClass,Global.top,
-                XmNdeleteResponse,XmDESTROY,
+                xmDialogShellWidgetClass, appshell,
+                XmNdeleteResponse, XmDESTROY,
                 XmNdefaultPosition, FALSE,
+                XmNfontList, fontlist1,
                 NULL);
         }
 
@@ -327,6 +346,7 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
                                       XmNleftOffset, 10,
                                       XmNrightAttachment, XmATTACH_NONE,
                                       XmNbackground, colors[0xff],
+                                      XmNfontList, fontlist1,
                                       NULL);
 
         locate_station_data = XtVaCreateManagedWidget("Locate_station data", xmTextFieldWidgetClass, form,
@@ -348,6 +368,7 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
                                       XmNrightOffset, 10,
                                       XmNnavigationType, XmTAB_GROUP,
                                       XmNtraversalOn, TRUE,
+                                      XmNfontList, fontlist1,
                                       NULL);
 
         locate_case_data  = XtVaCreateManagedWidget(langcode("WPUPLSP003"),xmToggleButtonWidgetClass,form,
@@ -361,6 +382,7 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
                                       XmNbackground, colors[0xff],
                                       XmNnavigationType, XmTAB_GROUP,
                                       XmNtraversalOn, TRUE,
+                                      XmNfontList, fontlist1,
                                       NULL);
 
         locate_match_data  = XtVaCreateManagedWidget(langcode("WPUPLSP004"),xmToggleButtonWidgetClass,form,
@@ -375,6 +397,7 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
                                       XmNbackground, colors[0xff],
                                       XmNnavigationType, XmTAB_GROUP,
                                       XmNtraversalOn, TRUE,
+                                      XmNfontList, fontlist1,
                                       NULL);
 
         sep = XtVaCreateManagedWidget("Locate_station sep", xmSeparatorGadgetClass,form,
@@ -386,6 +409,7 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
                                       XmNleftAttachment, XmATTACH_FORM,
                                       XmNrightAttachment,XmATTACH_FORM,
                                       XmNbackground, colors[0xff],
+                                      XmNfontList, fontlist1,
                                       NULL);
 
         button_lookup = XtVaCreateManagedWidget(langcode("WPUPLSP007"),xmPushButtonGadgetClass, form,
@@ -402,6 +426,7 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
                                       XmNbackground, colors[0xff],
                                       XmNnavigationType, XmTAB_GROUP,
                                       XmNtraversalOn, TRUE,
+                                      XmNfontList, fontlist1,
                                       NULL);
 
         button_locate = XtVaCreateManagedWidget(langcode("WPUPLSP005"),xmPushButtonGadgetClass, form,
@@ -418,6 +443,7 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
                                       XmNbackground, colors[0xff],
                                       XmNnavigationType, XmTAB_GROUP,
                                       XmNtraversalOn, TRUE,
+                                      XmNfontList, fontlist1,
                                       NULL);
 
         button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00002"),xmPushButtonGadgetClass, form,
@@ -434,6 +460,7 @@ begin_critical_section(&locate_station_dialog_lock, "locate_gui.c:Locate_station
                                       XmNbackground, colors[0xff],
                                       XmNnavigationType, XmTAB_GROUP,
                                       XmNtraversalOn, TRUE,
+                                      XmNfontList, fontlist1,
                                       NULL);
 
         XtAddCallback(button_lookup, XmNactivateCallback, fcc_rac_lookup, locate_station_dialog);
@@ -570,10 +597,12 @@ begin_critical_section(&locate_place_chooser_lock, "locate_gui.c:Locate_place_ch
     if (locate_place_chooser == NULL) {
 
         // Set up a selection box:
-        locate_place_chooser = XtVaCreatePopupShell(langcode("WPUPCFS028"),xmDialogShellWidgetClass,Global.top,
-            XmNdeleteResponse,XmDESTROY,
+        locate_place_chooser = XtVaCreatePopupShell(langcode("WPUPCFS028"),
+            xmDialogShellWidgetClass, appshell,
+            XmNdeleteResponse, XmDESTROY,
             XmNdefaultPosition, FALSE,
             XmNbackground, colors[0xff],
+            XmNfontList, fontlist1,
             NULL);
 
         pane = XtVaCreateWidget("Locate_place_chooser pane",xmPanedWindowWidgetClass, locate_place_chooser,
@@ -601,6 +630,7 @@ begin_critical_section(&locate_place_chooser_lock, "locate_gui.c:Locate_place_ch
             XmNrightAttachment, XmATTACH_POSITION,
             XmNrightPosition, 2,
             XmNnavigationType, XmTAB_GROUP,
+            XmNfontList, fontlist1,
             NULL);
 
         XtAddCallback(button_ok,
@@ -617,6 +647,7 @@ begin_critical_section(&locate_place_chooser_lock, "locate_gui.c:Locate_place_ch
             XmNrightAttachment, XmATTACH_POSITION,
             XmNrightPosition, 4,
             XmNnavigationType, XmTAB_GROUP,
+            XmNfontList, fontlist1,
             NULL);
 
         XtAddCallback(button_cancel,
@@ -641,6 +672,7 @@ begin_critical_section(&locate_place_chooser_lock, "locate_gui.c:Locate_place_ch
         XtSetArg(al[ac], XmNrightOffset, 5); ac++;
         XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
         XtSetArg(al[ac], XmNleftOffset, 5); ac++;
+        XtSetArg(al[ac], XmNfontList, fontlist1); ac++;
 
         locate_place_list = XmCreateScrolledList(form,"Locate_place_chooser list",al,ac);
 
@@ -768,18 +800,20 @@ void Locate_place_now(Widget w, XtPointer clientData, XtPointer callData) {
 
     /*fprintf(stderr,"looking for %s\n",locate_place_name);*/
 
-    match_quantity = gnis_locate_place(da,
-        locate_place_name,
-        locate_state_name,
-        locate_county_name,
-        locate_quad_name,
-        locate_type_name,
-        locate_gnis_filename,
+    match_quantity = gnis_locate_place(da, locate_place_name,
+        locate_state_name, locate_county_name, locate_quad_name,
+        locate_type_name, locate_gnis_filename,
         (int)XmToggleButtonGetState(locate_place_case_data),
         (int)XmToggleButtonGetState(locate_place_match_data),
-        match_array_name,
-        match_array_lat,
-        match_array_long);
+        match_array_name, match_array_lat, match_array_long);
+
+    if (0 == match_quantity) // Try population centers.
+	    match_quantity = pop_locate_place(da, locate_place_name,
+        	locate_state_name, locate_county_name, locate_quad_name,
+        	locate_type_name, locate_gnis_filename,
+		(int)XmToggleButtonGetState(locate_place_case_data),
+       		(int)XmToggleButtonGetState(locate_place_match_data),
+        	match_array_name, match_array_lat, match_array_long);
 
     if (match_quantity) {
         // Found some matches!
@@ -843,9 +877,10 @@ void Locate_place(/*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@u
 begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
 
         locate_place_dialog = XtVaCreatePopupShell(langcode("PULDNMP014"),
-                xmDialogShellWidgetClass,Global.top,
-                XmNdeleteResponse,XmDESTROY,
+                xmDialogShellWidgetClass, appshell,
+                XmNdeleteResponse, XmDESTROY,
                 XmNdefaultPosition, FALSE,
+                XmNfontList, fontlist1,
                 NULL);
 
         pane = XtVaCreateWidget("Locate_place pane",xmPanedWindowWidgetClass, locate_place_dialog,
@@ -867,6 +902,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNleftOffset, 10,
                 XmNrightAttachment, XmATTACH_NONE,
                 XmNbackground, colors[0xff],
+                XmNfontList, fontlist1,
                 NULL);
 
         locate_place_data = XtVaCreateManagedWidget("Locate_place_data", xmTextFieldWidgetClass, form,
@@ -888,6 +924,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNrightOffset, 10,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         state = XtVaCreateManagedWidget(langcode("FEATURE002"),xmLabelWidgetClass, form,
@@ -899,6 +936,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNleftOffset, 10,
                 XmNrightAttachment, XmATTACH_NONE,
                 XmNbackground, colors[0xff],
+                XmNfontList, fontlist1,
                 NULL);
 
         locate_state_data = XtVaCreateManagedWidget("Locate_state_data", xmTextFieldWidgetClass, form,
@@ -921,6 +959,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNrightOffset, 10,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         county = XtVaCreateManagedWidget(langcode("FEATURE003"),xmLabelWidgetClass, form,
@@ -932,6 +971,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNleftOffset, 10,
                 XmNrightAttachment, XmATTACH_NONE,
                 XmNbackground, colors[0xff],
+                XmNfontList, fontlist1,
                 NULL);
 
         locate_county_data = XtVaCreateManagedWidget("Locate_county_data", xmTextFieldWidgetClass, form,
@@ -954,6 +994,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNrightOffset, 10,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         quad = XtVaCreateManagedWidget(langcode("FEATURE004"),xmLabelWidgetClass, form,
@@ -965,6 +1006,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNleftOffset, 10,
                 XmNrightAttachment, XmATTACH_NONE,
                 XmNbackground, colors[0xff],
+                XmNfontList, fontlist1,
                 NULL);
 
         locate_quad_data = XtVaCreateManagedWidget("Locate_quad_data", xmTextFieldWidgetClass, form,
@@ -987,6 +1029,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNrightOffset, 10,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         place_type = XtVaCreateManagedWidget(langcode("FEATURE005"),xmLabelWidgetClass, form,
@@ -998,6 +1041,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNleftOffset, 10,
                 XmNrightAttachment, XmATTACH_NONE,
                 XmNbackground, colors[0xff],
+                XmNfontList, fontlist1,
                 NULL);
 
         locate_type_data = XtVaCreateManagedWidget("Locate_type_data", xmTextFieldWidgetClass, form,
@@ -1020,6 +1064,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNrightOffset, 10,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         gnis_file = XtVaCreateManagedWidget(langcode("FEATURE006"),xmLabelWidgetClass, form,
@@ -1031,6 +1076,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNleftOffset, 10,
                 XmNrightAttachment, XmATTACH_NONE,
                 XmNbackground, colors[0xff],
+                XmNfontList, fontlist1,
                 NULL);
 
         locate_gnis_file_data = XtVaCreateManagedWidget("locate_gnis_file_data", xmTextFieldWidgetClass, form,
@@ -1053,6 +1099,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNrightOffset, 10,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
 
@@ -1067,6 +1114,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNbackground, colors[0xff],
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         locate_place_match_data  = XtVaCreateManagedWidget(langcode("WPUPLSP004"),xmToggleButtonWidgetClass,form,
@@ -1081,6 +1129,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNbackground, colors[0xff],
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         sep = XtVaCreateManagedWidget("Locate_place sep", xmSeparatorGadgetClass,form,
@@ -1092,6 +1141,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNleftAttachment, XmATTACH_FORM,
                 XmNrightAttachment,XmATTACH_FORM,
                 XmNbackground, colors[0xff],
+                XmNfontList, fontlist1,
                 NULL);
 
         button_ok = XtVaCreateManagedWidget(langcode("WPUPLSP005"),xmPushButtonGadgetClass, form,
@@ -1108,6 +1158,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNbackground, colors[0xff],
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00002"),xmPushButtonGadgetClass, form,
@@ -1124,6 +1175,7 @@ begin_critical_section(&locate_place_dialog_lock, "locate_gui.c:Locate_place" );
                 XmNbackground, colors[0xff],
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         XtAddCallback(button_ok, XmNactivateCallback, Locate_place_now, locate_place_dialog);
