@@ -1,9 +1,9 @@
 /*
- * $Id: map_cache.c,v 1.16 2005/07/21 02:19:11 we7u Exp $
+ * $Id: map_cache.c,v 1.36 2013/01/18 17:17:25 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,7 +36,10 @@
 
 // Need this one before the #ifdef in order to get the definition of
 // USE_MAP_CACHE, if defined.
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 
 #ifdef  USE_MAP_CACHE
 //#warning USE_MAP_CACHE Defined
@@ -57,12 +60,24 @@
 #include "map_cache.h" 
 #include <db.h>
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
+
 
 // This is used to temporarily disable fetching from the map cache.
 // Used for refreshing corrupted maps in the cache.
 int map_cache_fetch_disable = 0;
 
 
+// Used to disable map caching entirely if the header and dblib
+// versions don't match, in order to avoid segfaults when the map
+// caching is used.
+int map_cache_disabled = 0;
+
+
 
 
 
@@ -79,7 +94,17 @@ void map_cache_init(void) {
             "\n\n***** WARNING *****\n");
         fprintf(stderr,
             "Berkeley DB header files/shared library file do NOT match!\n");
+        fprintf(stderr,
+            "Disabling use of the map cache.\n");
+
+
+// Can't bring up a popup here 'cuz we don't have any GUI running
+// yet by this point.
+// popup_message_always(langcode("POPEM00034"),langcode("POPEM00046"));
+
+
         warn_now++;
+        map_cache_disabled++;
     }
 
     if (debug_level & 5 || warn_now) {
@@ -126,14 +151,18 @@ int map_cache_put( char * map_cache_url, char * map_cache_file ){
     DB *dbp; 
     struct stat file_status;
     char mc_buf[128];
+    char temp_file_path[MAX_VALUE];
 
+    if (map_cache_disabled) {
+        return(1);
+    }
 
     mc_space_used=0; 
 
     xastir_snprintf(mc_database_filename,
     sizeof(mc_database_filename),
     "%s/map_cache.db", 
-    get_user_base_dir("map_cache"));
+    get_user_base_dir("map_cache", temp_file_path, sizeof(temp_file_path)));
 
     // check for reasonable filename
     // expects file name like /home/brown/.xastir/map_cache/map_1100052372.gif
@@ -187,7 +216,7 @@ int map_cache_put( char * map_cache_url, char * map_cache_file ){
         dbp->err(dbp, mc_ret, "%s", mc_database_filename);
         db_strerror(mc_ret); 
     }
-#elif	 (DB_VERSION_MAJOR==4 && DB_VERSION_MINOR>=1 )
+#else
 
     if ((mc_ret = dbp->open(dbp,
             NULL,mc_database_filename, NULL, DB_CREATE, DB_BTREE, 0664)) != 0) {
@@ -211,11 +240,15 @@ int map_cache_put( char * map_cache_url, char * map_cache_file ){
     mc_key.size = sizeof("CACHE_SPACE_USED"); 
 
 // check "CACHE_SPACE_USED" record in db
-	
-    if (((mc_ret = dbp->get(dbp, NULL, &mc_key, &mc_data, 0)) == 0) &&  ( mc_data.data != NULL ) ) {
-        ( debug_level & 512 ) && fprintf(stderr, "map_cache_put: %s: key retrieved: data was %s.\n",
-            (mc_key.data == NULL) ? "(null)" : (char *)mc_key.data,
-            (mc_data.data == NULL) ? "(null)" : (char *)mc_data.data);
+
+// mc_ret is assigned here but not used.  Commented it out for now.
+    if (((/* mc_ret = */ dbp->get(dbp, NULL, &mc_key, &mc_data, 0)) == 0) &&  ( mc_data.data != NULL ) ) {
+
+        if ( debug_level & 512 ) {
+            fprintf(stderr, "map_cache_put: %s: key retrieved: data was %s.\n",
+                (mc_key.data == NULL) ? "(null)" : (char *)mc_key.data,
+                (mc_data.data == NULL) ? "(null)" : (char *)mc_data.data);
+        }
 
         if (mc_data.data == NULL)
             mc_space_used = 0;
@@ -349,7 +382,10 @@ int map_cache_put( char * map_cache_url, char * map_cache_file ){
         // db_strerror(mc_ret); 
         return(mc_ret); 
     } 
-    statusline("Map now cached",1);
+
+    // Map now cached
+    statusline(langcode("CACHE001"), 1);
+
 // close the db 
 
     // Only try the close if we have a valid handle
@@ -371,6 +407,9 @@ int map_cache_put( char * map_cache_url, char * map_cache_file ){
 
 // map_cache_get()
 //
+// Queries URL->Filename db 
+// Calls map_cache_del to cleanup expired maps 
+//
 // Inputs:
 //
 // Outputs: 0 if cached file is retrieved successfully
@@ -380,22 +419,22 @@ int map_cache_put( char * map_cache_url, char * map_cache_file ){
 //          return value from dbp->get if record not found
 //
 int map_cache_get( char * map_cache_url, char * map_cache_file ){
-
-// Queries URL->Filename db 
-// Calls map_cache_del to cleanup expired maps 
-
     DBT mc_key, mc_data ; 
     DB *dbp;
     int mc_ret, mc_t_ret, mc_file_stat ;
     char mc_database_filename[MAX_FILENAME]; 
     struct stat file_status;
+    char temp_file_path[MAX_VALUE];
 
+    if (map_cache_disabled) {
+        return(1);
+    }
 
 set_dangerous("map_cache_get: xastir_snprintf 1");
     xastir_snprintf(mc_database_filename,
         sizeof(mc_database_filename),   // change to max_filename?
         "%s/map_cache.db",
-        get_user_base_dir("map_cache"));
+        get_user_base_dir("map_cache", temp_file_path, sizeof(temp_file_path)));
 clear_dangerous();
  
 set_dangerous("map_cache_get: db_create");
@@ -413,18 +452,20 @@ clear_dangerous();
 set_dangerous("map_cache_get:dbp->open 1");
     if ((mc_ret = dbp->open(dbp,
         mc_database_filename, NULL, DB_CREATE, DB_BTREE, 0664)) != 0) {
-        ( debug_level & 512 ) ?  dbp->err(dbp, mc_ret, "%s", mc_database_filename):0;
-        // db_strerror(mc_ret);
+            ( debug_level & 512 ) ?  dbp->err(dbp, mc_ret, "%s", mc_database_filename):0;
+            // db_strerror(mc_ret);
     }
 clear_dangerous();
 
-#elif	 (DB_VERSION_MAJOR==4 && DB_VERSION_MINOR>=1 )
+#else
 
 set_dangerous("map_cache_get:dbp->open 2");
     if ((mc_ret = dbp->open(dbp,
-        NULL,mc_database_filename, NULL, DB_CREATE, DB_BTREE, 0664)) != 0) {
-        ( debug_level & 512 ) ? dbp->err(dbp, mc_ret, "%s", mc_database_filename) : 0 ;
-        // db_strerror(mc_ret);
+            NULL,mc_database_filename, NULL, DB_CREATE, DB_BTREE, 0664)) != 0) {
+        if (debug_level & 512) {
+            dbp->err(dbp, mc_ret, "%s", mc_database_filename);
+            // db_strerror(mc_ret);
+        }
     }
 clear_dangerous();
 
@@ -469,6 +510,12 @@ clear_dangerous();
             }
 set_dangerous("map_cache_get: map_cache_del 1");
             map_cache_del(map_cache_url);
+            if (dbp != NULL) {
+                if ((mc_t_ret = dbp->close(dbp, 0)) != 0 && mc_ret == 0){
+                    mc_ret = mc_t_ret;
+                   // db_strerror(mc_ret);
+                }
+            }
 clear_dangerous();
   
             return (-1 * mc_ret); 
@@ -489,6 +536,12 @@ set_dangerous("map_cache_get: map_cache_expired");
             }
 set_dangerous("map_cache_get: map_cache_del 2");
             map_cache_del(map_cache_url);
+            if (dbp != NULL) {
+                if ((mc_t_ret = dbp->close(dbp, 0)) != 0 && mc_ret == 0){
+                    mc_ret = mc_t_ret;
+                   // db_strerror(mc_ret);
+                }
+            }
 clear_dangerous();
             return (mc_ret);
         } 
@@ -556,7 +609,9 @@ set_dangerous("map_cache_get: dbp->close 2");
             }
 clear_dangerous();
             // If we made it here all is good
-            statusline("Loading Cached Map",1);
+
+            // Loading Cached Map
+            statusline(langcode("CACHE002"), 1);
             return (0); 
         }
 		
@@ -567,13 +622,25 @@ clear_dangerous();
                 (mc_key.data == NULL) ? "(null)" : (char *)mc_key.data);
         }
 
-        ( debug_level & 512 ) ? dbp->err(dbp, mc_ret, "DB->get"):0;
-        // db_strerror(mc_ret);
+        if (debug_level & 512) {
+            dbp->err(dbp, mc_ret, "DB->get");
+            // db_strerror(mc_ret);
+        }
 
         // there was some problem getting things from the db
         // return the return from the get 
 
-        statusline("Map not found in cache...",1);        
+        // Map not found in cache...
+        statusline(langcode("CACHE003"), 1);
+	set_dangerous("map_cache_get: dbp->close 3");
+	// Only try the close if we have a valid handle
+	if (dbp != NULL) {
+	    if ((mc_t_ret = dbp->close(dbp, 0)) != 0 && mc_ret == 0){
+		mc_ret = mc_t_ret;
+		// db_strerror(mc_ret);
+	    }
+	}
+	clear_dangerous();
         return (mc_ret); 
     }
 
@@ -604,14 +671,18 @@ int map_cache_del( char * map_cache_url ){
     char mc_delete_file[MAX_FILENAME]; 
     struct stat file_status;
     char mc_buf[128];
+    char temp_file_path[MAX_VALUE];
 
+    if (map_cache_disabled) {
+        return(1);
+    }
 
     mc_space_used = 0 ; 
 
     xastir_snprintf(mc_database_filename,
         MAX_FILENAME,
         "%s/map_cache.db",
-        get_user_base_dir("map_cache"));
+        get_user_base_dir("map_cache", temp_file_path, sizeof(temp_file_path)));
 
     if ((mc_ret = db_create(&dbp, NULL, 0)) != 0) {
         fprintf(stderr, "map_cache_del db_create:%s\n", db_strerror(mc_ret)); 
@@ -628,12 +699,14 @@ int map_cache_del( char * map_cache_url ){
         // db_strerror(mc_ret);
         return(1);
     }
-#elif	 (DB_VERSION_MAJOR==4 && DB_VERSION_MINOR>=1 )
+#else
 	
     if ((mc_ret = dbp->open(dbp,
-        NULL,mc_database_filename, NULL, DB_CREATE, DB_BTREE, 0664)) != 0) {
-        ( debug_level & 512 ) ? dbp->err(dbp, mc_ret, "%s", mc_database_filename):0;
-        // db_strerror(mc_ret);
+            NULL,mc_database_filename, NULL, DB_CREATE, DB_BTREE, 0664)) != 0) {
+        if (debug_level & 512) {
+            dbp->err(dbp, mc_ret, "%s", mc_database_filename);
+            // db_strerror(mc_ret);
+        }
         return(1);
     }
 
@@ -871,7 +944,7 @@ clear_dangerous();
 
 
 
-char * map_cache_fileid(){
+char * map_cache_fileid(void) {
 
     // returns a unique identifier 
     // used for generating filenames for cached files
@@ -889,19 +962,22 @@ char * map_cache_fileid(){
 
 
 
+// check for files old enough to expire 
+// this is lame but it should work for now. 
+// expects file name like /home/brown/.xastir/map_cache/map_1100052372.gif
+//
+// writing this proved a good example of why pointer arithmetic is tricky
+// and a good example of why I should avoid it - n8ysz 20041110
+// 
 int map_cache_expired( char * mc_filename, time_t mc_max_age ){
-
-    // check for files old enough to expire 
-    // this is lame but it should work for now. 
-    // expects file name like /home/brown/.xastir/map_cache/map_1100052372.gif
-
-    // writing this proved a good example of why pointer arithmetic is tricky
-    // and a good example of why I should avoid it - n8ysz 20041110
-    
     time_t mc_t,mc_file_age; 
     char *mc_filename_tmp, *mc_time_buf_tmp, *mc_time_buf;
-    
-    
+
+
+    if (map_cache_disabled) {
+        return(0);
+    }
+
     mc_time_buf=malloc(MAX_FILENAME);
     mc_time_buf_tmp=mc_time_buf; 
 
@@ -955,20 +1031,26 @@ int map_cache_expired( char * mc_filename, time_t mc_max_age ){
 
 // Functions that need writing 
 
-int mc_check_space_used () {
-    
-   return(0); 
+int mc_check_space_used (void) {
 
+    if (map_cache_disabled) {
+        return(0);
+    }
+    
+    return(0); 
 }
 
 
 
 
 
-int mc_update_space_used () {
-    
-   return(0); 
+int mc_update_space_used (void) {
 
+    if (map_cache_disabled) {
+        return(0);
+    }
+    
+    return(0); 
 }
 
 
