@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_dos.c,v 1.22 2006/04/24 19:21:41 we7u Exp $
+ * $Id: map_dos.c,v 1.30 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2006  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -161,8 +161,8 @@ void map_plot (Widget w, long max_x, long max_y, long x_long_cord,
     }
 
     if (draw_ok) {
-        x = ((x_long_cord - x_long_offset) / scale_x);
-        y = ((y_lat_cord  - y_lat_offset)  / scale_y);
+        x = ((x_long_cord - NW_corner_longitude) / scale_x);
+        y = ((y_lat_cord  - NW_corner_latitude)  / scale_y);
         if (x < -MAX_OUTBOUND)
             x = -MAX_OUTBOUND;
 
@@ -261,11 +261,21 @@ void map_plot (Widget w, long max_x, long max_y, long x_long_cord,
                 switch (destination_pixmap) {
 
                     case DRAW_TO_PIXMAP_FINAL:
-                        (void)XDrawLines (XtDisplay (w), pixmap_final, gc, points, npoints,CoordModeOrigin);
+                        (void)XDrawLines (XtDisplay (w),
+                            pixmap_final,
+                            gc,
+                            points,
+                            l16(npoints),
+                            CoordModeOrigin);
                         break;
 
                     case DRAW_TO_PIXMAP:
-                        (void)XDrawLines (XtDisplay (w), pixmap, gc, points, npoints,CoordModeOrigin);
+                        (void)XDrawLines (XtDisplay (w),
+                            pixmap,
+                            gc,
+                            points,
+                            l16(npoints),
+                            CoordModeOrigin);
                         break;
 
                     case DRAW_TO_PIXMAP_ALERTS:
@@ -287,8 +297,8 @@ void map_plot (Widget w, long max_x, long max_y, long x_long_cord,
                 npoints = 0;
                 first_behavior = (unsigned char)object_behavior;
             }
-            points[npoints].x = (short)x;
-            points[npoints].y = (short)y;
+            points[npoints].x = l16(x);
+            points[npoints].y = l16(y);
             if (    (points[npoints].x > (-MAX_OUTBOUND))
                  && (points[npoints].x < (short)max_x)
                  && (points[npoints].y > (-MAX_OUTBOUND))
@@ -300,8 +310,8 @@ void map_plot (Widget w, long max_x, long max_y, long x_long_cord,
             last_behavior = (unsigned char)object_behavior;
             return;
         }
-        points[npoints].x = (short)x;
-        points[npoints].y = (short)y;
+        points[npoints].x = l16(x);
+        points[npoints].y = l16(y);
         last_behavior = (unsigned char)object_behavior;
 
         if (points[npoints].x != points[npoints - 1].x || points[npoints].y != points[npoints - 1].y) {
@@ -344,7 +354,7 @@ void draw_dos_map(Widget w,
     char file_name[33];
 //    char *ext;
     char map_title[33];
-    char map_creator[8];
+    char map_creator[9];
     unsigned long creation_date;
     unsigned long left_boundary;
     unsigned long right_boundary;
@@ -1129,11 +1139,11 @@ process:        if (strncasecmp ("Line", map_version, 4) == 0) {
                         label_mag = (int)strtol (trailer, &trailer, 0) * 20;
 
                         if ((label_type[0] & 0x80) == '\0') /* left of coords */
-                            x = ((label_x_cord - x_long_offset) / scale_x) - (label_length * 6);
+                            x = ((label_x_cord - NW_corner_longitude) / scale_x) - (label_length * 6);
                         else  /* right of coords */
-                            x = ((label_x_cord - x_long_offset) / scale_x);
+                            x = ((label_x_cord - NW_corner_longitude) / scale_x);
                 
-                        y = ((label_y_cord - y_lat_offset) / scale_y);
+                        y = ((label_y_cord - NW_corner_latitude) / scale_y);
                         if (x > (0) && (x < (int)screen_width)) {
                             if (y > (0) && (y < (int)screen_height)) {
                                 /*fprintf(stderr,"Label mag %d mag %d\n",label_mag,(scale_x*2)-1); */
@@ -1240,15 +1250,15 @@ process:        if (strncasecmp ("Line", map_version, 4) == 0) {
               
                     if ((label_type_1[0] & 0x80) == '\0') {
                         /* left of coords */
-                        x = ((label_x_cord - x_long_offset) / scale_x) - (label_length * 6);
+                        x = ((label_x_cord - NW_corner_longitude) / scale_x) - (label_length * 6);
                         x = 0;  // ??????
                     }
                     else {
                         /* right of coords */
-                        x = ((label_x_cord - x_long_offset) / scale_x);
+                        x = ((label_x_cord - NW_corner_longitude) / scale_x);
                     }
               
-                    y = ((label_y_cord - y_lat_offset) / scale_y);
+                    y = ((label_y_cord - NW_corner_latitude) / scale_y);
               
                     if (x > (0) && (x < (int)screen_width)) {
 
@@ -1314,8 +1324,8 @@ process:        if (strncasecmp ("Line", map_version, 4) == 0) {
                     if (label_text_color < '1' && label_text_color > '9')
                         label_text_color = '0'; // Default to black
               
-                    x = ((label_x_cord - x_long_offset) / scale_x);
-                    y = ((label_y_cord - y_lat_offset) / scale_y);
+                    x = ((label_x_cord - NW_corner_longitude) / scale_x);
+                    y = ((label_y_cord - NW_corner_latitude) / scale_y);
               
                     // Read the label text portion
                     if (strcmp (map_type, "COMP") == 0) {
