@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_gnis.c,v 1.21 2005/07/31 06:19:14 we7u Exp $
+ * $Id: map_gnis.c,v 1.41 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,10 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -36,7 +39,9 @@
 #include <errno.h>
 
 // Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
 
 #include <dirent.h>
 #include <netinet/in.h>
@@ -68,6 +73,11 @@
 #include "color.h"
 #include "xa_config.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
 #define CHECKMALLOC(m)  if (!m) { fprintf(stderr, "***** Malloc Failed *****\n"); exit(0); }
 
 //NOTE:  This function has a problem if a non-gnis file is labeled
@@ -85,7 +95,7 @@
 //
 // These types of files are available from http://geonames.usgs.gov/
 // under "Download State Gazetteer Data - Available Via Anonymous FTP".
-// A typical filename would be: "WA.deci.gz".   Do not get the other
+// A typical filename would be: "WA_Features_20090401.zip".   Do not get the other
 // types of files which are columnar.  The files that we parse are
 // pipe-delimited.
 //
@@ -130,6 +140,7 @@ void draw_gnis_map (Widget w,
     unsigned long left_extent = 0l;
     unsigned long right_extent = 0l;
     char status_text[MAX_FILENAME];
+    int count = 0;
 
 
     //fprintf(stderr,"draw_gnis_map starting: %s/%s\n",dir,filenm);
@@ -156,10 +167,10 @@ void draw_gnis_map (Widget w,
     }
 
     // Screen view
-    min_lat = y_lat_offset + (long)(screen_height * scale_y);
-    max_lat = y_lat_offset;
-    min_lon = x_long_offset;
-    max_lon = x_long_offset + (long)(screen_width  * scale_x);
+    min_lat = SE_corner_latitude;
+    max_lat = NW_corner_latitude;
+    min_lon = NW_corner_longitude;
+    max_lon = SE_corner_longitude;
 
 
     // The map extents in the map index are checked in draw_map to
@@ -187,7 +198,16 @@ void draw_gnis_map (Widget w,
     HandlePendingEvents(app_context);
     if (interrupt_drawing_now) {
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -221,6 +241,31 @@ Cell Name
     f = fopen (file, "r");
     if (f != NULL) {
         while (!feof (f)) {     // Loop through entire file
+            int lat_in_view = 0;
+
+            count++;
+            if ((count % 16) == 0) {
+
+                // Check whether map drawing should be interrupted.
+                // Check every 16 lines.
+                //
+                HandlePendingEvents(app_context);
+                if (interrupt_drawing_now) {
+                    (void)fclose (f);
+                    // Update to screen
+                    (void)XCopyArea(XtDisplay(da),
+                        pixmap,
+                        XtWindow(da),
+                        gc,
+                        0,
+                        0,
+                        (unsigned int)screen_width,
+                        (unsigned int)screen_height,
+                        0,
+                        0);
+                    return;
+                }
+            }
 
             if ( get_line (f, line, MAX_FILENAME) ) {  // Snag one line of data
 
@@ -230,6 +275,17 @@ Cell Name
 
                 if (strlen(line) > 0) {
 
+                    // Default population, in case the field isn't
+                    // present in the file.
+                    xastir_snprintf(population,sizeof(population),"0");
+ 
+// Examples of old/new format:
+// 1462331|VA|Abingdon Elementary School|school|Arlington|51|013|385023N|0770546W|38.83972|-77.09611||||||||Alexandria
+// 1462331|VA|Abingdon Elementary School|School|Arlington|51|013|385023N|0770545W|38.8398349|-77.0958117|56|Alexandria
+// 2008 Format follows
+//FEATURE_ID|FEATURE_NAME|FEATURE_CLASS|STATE_ALPHA|STATE_NUMERIC|COUNTY_NAME|COUNTY_NUMERIC|PRIMARY_LAT_DMS|PRIM_LONG_DMS|PRIM_LAT_DEC|PRIM_LONG_DEC|SOURCE_LAT_DMS|SOURCE_LONG_DMS|SOURCE_LAT_DEC|SOURCE_LONG_DEC|ELEVATION|MAP_NAME|DATE_CREATED|DATE_EDITED
+//205110|Appalachian National Scenic Trail|Trail|PA|42|Perry|099|401920N|0770439W|40.3221113|-77.0775473|||||201|Wertzville|09/12/1979|05/19/2008
+
                     //NOTE:  We handle running off the end of "line"
                     //via the "continue" statement.  Skip the line
                     //if we don't find enough parameters while
@@ -242,64 +298,41 @@ Cell Name
                         continue;   // Skip this line
                     }
 
-                    // Find end of State field
-                    i = index(j+1, '|');
-
-                    if (i == NULL) {    // Pipe not found
-                        continue;   // Skip this line
-                    }
-
-                    i[0] = '\0';
-                    xastir_snprintf(state,sizeof(state),"%s",j+1);
-                    clean_string(state);
-
 //NOTE:  It'd be nice to take the part after the comma and put it before the rest
 // of the text someday, i.e. "Cassidy, Lake".
 
                     // Find end of Feature Name field
-                    j = index(i+1, '|');
+                    i = index(++j, '|');
 
-                    if (j == NULL) {    // Pipe not found
+                    if (i == NULL) {    // Pipe not found
                         continue;   // Skip this line
                     }
 
-                    j[0] = '\0';
-                    xastir_snprintf(name,sizeof(name),"%s",i+1);
-                    clean_string(name);
+                    i[0] = '\0';
+                    xastir_snprintf(name,sizeof(name),"%s",j);
 
                     // Find end of Feature Type field
-                    i = index(j+1, '|');
-
-                    if (i == NULL) {    // Pipe not found
-                        continue;   // Skip this line
-                    }
-
-                    i[0] = '\0';
-                    xastir_snprintf(type,sizeof(type),"%s",j+1);
-                    clean_string(type);
-
-                    // Find end of County Name field
-                    j = index(i+1, '|');
+                    j = index(++i, '|');
 
                     if (j == NULL) {    // Pipe not found
                         continue;   // Skip this line
                     }
 
                     j[0] = '\0';
-                    xastir_snprintf(county,sizeof(county),"%s",i+1);
-                    clean_string(county);
+                    xastir_snprintf(type,sizeof(type),"%s",i);
+
+                    // Find end of State field
+                    i = index(++j, '|');
+
+                    if (i == NULL) {    // Pipe not found
+                        continue;   // Skip this line
+                    }
+
+                    i[0] = '\0';
+                    xastir_snprintf(state,sizeof(state),"%s",j);
 
                     // Find end of State Number Code field
-                    i = index(j+1, '|');
-
-                    if (i == NULL) {    // Pipe not found
-                        continue;   // Skip this line
-                    }
-
-                    i[0] = '\0';
-
-                    // Find end of County Number Code field
-                    j = index(i+1, '|');
+                    j = index(++i, '|');
 
                     if (j == NULL) {    // Pipe not found
                         continue;   // Skip this line
@@ -307,121 +340,153 @@ Cell Name
 
                     j[0] = '\0';
 
-                    // Find end of Primary Latitude field (DDMMSSN)
-                    i = index(j+1, '|');
+                    // Find end of County Name field
+                    i = index(++j, '|');
 
                     if (i == NULL) {    // Pipe not found
                         continue;   // Skip this line
                     }
 
                     i[0] = '\0';
-                    xastir_snprintf(latitude,sizeof(latitude),"%s",j+1);
-                    if (!isdigit(latitude[0])) { // skip record if not
+                    xastir_snprintf(county,sizeof(county),"%s",j);
+
+                    // Find end of County Number Code field
+                    j = index(++i, '|');
+
+                    if (j == NULL) {    // Pipe not found
+                        continue;   // Skip this line
+                    }
+
+                    j[0] = '\0';
+
+// Examples of old/new format:
+// 1462331|VA|Abingdon Elementary School|school|Arlington|51|013|385023N|0770546W|38.83972|-77.09611||||||||Alexandria
+// 1462331|VA|Abingdon Elementary School|School|Arlington|51|013|385023N|0770545W|38.8398349|-77.0958117|56|Alexandria
+
+                    // Find end of Primary Latitude field (DDMMSSN)
+                    i = index(++j, '|');
+
+                    if (i == NULL) {    // Pipe not found
+                        continue;   // Skip this line
+                    }
+
+                    i[0] = '\0';
+                    xastir_snprintf(latitude,sizeof(latitude),"%s",j);
+                    if (!isdigit((int)latitude[0])) { // skip record if not
                         continue;                // numeric! (e.g. "UNKNOWN")
                     }
 
+//WE7U
                     clean_string(latitude);
                     
                     // Find end of Primary Longitude field (DDDMMSSW)
-                    j = index(i+1, '|');
+                    j = index(++i, '|');
 
                     if (j == NULL) {    // Pipe not found
                         continue;   // Skip this line
                     }
 
                     j[0] = '\0';
-                    xastir_snprintf(longitude,sizeof(longitude),"%s",i+1);
-                    if (!isdigit(longitude[0])) { // skip record if not
+                    xastir_snprintf(longitude,sizeof(longitude),"%s",i);
+                    if (!isdigit((int)longitude[0])) { // skip record if not
                         continue;                 // numeric (e.g. UNKNOWN)
                     }
 
+//WE7U
                     clean_string(longitude);
 
                     // Find end of Primary Latitude field (decimal
                     // degrees)
-                    i = index(j+1, '|');
+                    i = index(++j, '|');
 
                     if (i == NULL) {    // Pipe not found
-                        continue;   // Skip this line
+                        goto FINISH;    // We have enough to process now
                     }
 
                     i[0] = '\0';
 
                     // Find end of Primary Longitude field (decimal
                     // degrees)
-                    j = index(i+1, '|');
+                    j = index(++i, '|');
 
                     if (j == NULL) {    // Pipe not found
-                        continue;   // Skip this line
+                        goto FINISH;    // We have enough to process now
                     }
 
                     j[0] = '\0';
 
-                    // Find end of Source Latitude field (DMS)
-                    i = index(j+1, '|');
+// Examples of old/new format:
+// 1462331|VA|Abingdon Elementary School|school|Arlington|51|013|385023N|0770546W|38.83972|-77.09611||||||||Alexandria
+// 1462331|VA|Abingdon Elementary School|School|Arlington|51|013|385023N|0770545W|38.8398349|-77.0958117|56|Alexandria
+ 
+                    // Find end of Source Latitude field (DMS) (old
+                    // format) or elevation (new format)
+                    i = index(++j, '|');
 
                     if (i == NULL) {    // Pipe not found
-                        continue;   // Skip this line
+                        goto FINISH;    // We have enough to process now
                     }
 
                     i[0] = '\0';
 
-                    // Find end of Source Longitude field (DMS)
-                    j = index(i+1, '|');
+                    // Find end of Source Longitude field (DMS) (old
+                    // format) or 
+                    j = index(++i, '|');
 
                     if (j == NULL) {    // Pipe not found
-                        continue;   // Skip this line
+                        goto FINISH;    // We have enough to process now
                     }
 
                     j[0] = '\0';
 
                     // Find end of Source Latitude field (decimal
                     // degrees)
-                    i = index(j+1, '|');
+                    i = index(++j, '|');
 
                     if (i == NULL) {    // Pipe not found
-                        continue;   // Skip this line
+                        goto FINISH;    // We have enough to process now
                     }
 
                     i[0] = '\0';
 
                     // Find end of Source Longitude field (decimal
                     // degrees)
-                    j = index(i+1, '|');
+                    j = index(++i, '|');
 
                     if (j == NULL) {    // Pipe not found
-                        continue;   // Skip this line
+                        goto FINISH;    // We have enough to process now
                     }
 
                     j[0] = '\0';
 
                     // Find end of Elevation field
-                    i = index(j+1, '|');
+                    i = index(++j, '|');
 
                     if (i == NULL) {    // Pipe not found
-                        continue;   // Skip this line
+                        goto FINISH;    // We have enough to process now
                     }
 
                     i[0] = '\0';
 
                     // Find end of Estimated Population field
-                    j = index(i+1, '|');
+                    j = index(++i, '|');
 
                     if (j == NULL) {    // Pipe not found
-                        continue;   // Skip this line
+                        goto FINISH;    // We have enough to process now
                     }
 
                     if ( j != NULL ) {
                         j[0] = '\0';
-                        xastir_snprintf(population,sizeof(population),"%s",i+1);
-                    } else {
-                        xastir_snprintf(population,sizeof(population),"0");
-                    } 
-                    clean_string(population);
+                        xastir_snprintf(population,sizeof(population),"%s",i);
+                    }
 
-                    // There are two more field, "Federal Status"
-                    // and "Cell Name".  We ignore those for now.
+FINISH:
+                    // There are two more fields (old format),
+                    // "Federal Status" and "Cell Name".  We ignore
+                    // those for now.
  
+		    if (strlen(latitude) < 7) 
+			continue;	// We really don't have any latitude here.
                     lat_dd[0] = latitude[0];
                     lat_dd[1] = latitude[1];
                     lat_dd[2] = '\0';
@@ -437,6 +502,40 @@ Cell Name
                     lat_dir[0] = latitude[6];
                     lat_dir[1] = '\0';
 
+                    // Now must convert from DD MM SS format to DD MM.MM format so that we
+                    // can run it through our conversion routine to Xastir coordinates.
+                    if (1 != sscanf(lat_ss, "%d", &temp1)) {
+                        fprintf(stderr,"draw_gnis_map:sscanf parsing error\n");
+                    }
+
+                    temp1 = (int)((temp1 / 60.0) * 100 + 0.5);  // Poor man's rounding
+                    xastir_snprintf(lat_str, sizeof(lat_str), "%s%s.%02d%s", lat_dd,
+                            lat_mm, temp1, lat_dir);
+                    coord_lat = convert_lat_s2l(lat_str);
+
+                    // Quick test of latitude to see if it's within
+                    // our view.  If not and we're not doing
+                    // indexing, skip this line and go on to the
+                    // next.
+                    if (coord_lat <= min_lat && coord_lat >= max_lat) {
+                        // Latitude is ok
+                        lat_in_view++;
+                    }
+                    else {  // Latitude not in current view
+
+                        // Check whether we're indexing the map
+                        if ( (destination_pixmap == INDEX_CHECK_TIMESTAMPS)
+                                || (destination_pixmap == INDEX_NO_TIMESTAMPS) ) {
+                            // Process the line 'cuz we're indexing
+                        }
+                        else {  // Not indexing so skip to the next
+                                // line in the file
+                            continue;
+                        }
+                    }
+
+		    if (strlen(longitude) < 8) 
+			continue;	// We really don't have any longitude here.
                     long_dd[0] = longitude[0];
                     long_dd[1] = longitude[1];
                     long_dd[2] = longitude[2];
@@ -453,17 +552,6 @@ Cell Name
                     long_dir[0] = longitude[7];
                     long_dir[1] = '\0';
 
-                    // Now must convert from DD MM SS format to DD MM.MM format so that we
-                    // can run it through our conversion routine to Xastir coordinates.
-                    if (1 != sscanf(lat_ss, "%d", &temp1)) {
-                        fprintf(stderr,"draw_gnis_map:sscanf parsing error\n");
-                    }
-
-                    temp1 = (int)((temp1 / 60.0) * 100 + 0.5);  // Poor man's rounding
-                    xastir_snprintf(lat_str, sizeof(lat_str), "%s%s.%02d%s", lat_dd,
-                            lat_mm, temp1, lat_dir);
-                    coord_lat = convert_lat_s2l(lat_str);
-
                     if (1 != sscanf(long_ss, "%d", &temp1)) {
                         fprintf(stderr,"draw_gnis_map:sscanf parsing error\n");
                     }
@@ -474,30 +562,35 @@ Cell Name
                     coord_lon = convert_lon_s2l(long_str);
 
 
-                    // Save the min/max extents of the file.  We
-                    // should really initially set the extents to
-                    // the min/max for the Xastir coordinate system,
-                    // but in practice zeroes should work just as
-                    // well.
-                    if ((coord_lat > (long)bottom_extent) || (bottom_extent == 0l))
-                        bottom_extent = coord_lat;
-                    if ((coord_lat < (long)top_extent) || (top_extent == 0l))
-                        top_extent = coord_lat;
-                    if ((coord_lon < (long)left_extent) || (left_extent == 0l))
-                        left_extent = coord_lon;
-                    if ((coord_lon > (long)right_extent) || (right_extent == 0l))
-                        right_extent = coord_lon;
-
-
                     // Check whether we're indexing the map
                     if ( (destination_pixmap == INDEX_CHECK_TIMESTAMPS)
                             || (destination_pixmap == INDEX_NO_TIMESTAMPS) ) {
-                        // Do nothing, we're indexing, not drawing
+
+                        // Save the min/max extents of the file.  We
+                        // should really initially set the extents
+                        // to the min/max for the Xastir coordinate
+                        // system, but in practice zeroes should
+                        // work just as well.
+                        //
+                        if ((coord_lat > (long)bottom_extent) || (bottom_extent == 0l))
+                            bottom_extent = coord_lat;
+                        if ((coord_lat < (long)top_extent) || (top_extent == 0l))
+                            top_extent = coord_lat;
+                        if ((coord_lon < (long)left_extent) || (left_extent == 0l))
+                            left_extent = coord_lon;
+                        if ((coord_lon > (long)right_extent) || (right_extent == 0l))
+                            right_extent = coord_lon;
                     }
                     // Now check whether this lat/lon is within our viewport.  If it
                     // is, draw a text label at that location.
                     else if (coord_lon >= min_lon && coord_lon <= max_lon
-                            && coord_lat <= min_lat && coord_lat >= max_lat) {
+                            && lat_in_view) {
+
+                        clean_string(state);
+                        clean_string(name);
+                        clean_string(type);
+                        clean_string(county);
+                        clean_string(population);
 
                         if (debug_level & 16) {
                             fprintf(stderr,"%s\t%s\t%s\t%s\t%s\t%s\t\t",
@@ -511,16 +604,12 @@ Cell Name
                         // here!  The format conversions you'll need
                         // if you try to compress this into two
                         // lines will get you into trouble.
-                        x = coord_lon - x_long_offset;
-                        y = coord_lat - y_lat_offset;
-                        x = x / scale_x;
-                        y = y / scale_y;
+                        x = coord_lon - NW_corner_longitude;
+                        y = coord_lat - NW_corner_latitude;
+                        x /= scale_x;
+                        y /= scale_y;
 
                         ok = 1;
-                        if (x >  16000) ok = 0;     // Skip this point
-                        if (x < -16000) ok = 0;     // Skip this point
-                        if (y >  16000) ok = 0;     // Skip this point
-                        if (y < -16000) ok = 0;     // Skip this point
 
                         /* set default symbol */
                         symbol_table = '/';
@@ -1005,64 +1094,64 @@ int gnis_locate_place( Widget w,
                     continue;   // Skip this line
                 }
 
-                // Find end of State field
-                i = index(j+1,'|');
-
-                if (i == NULL) {    // Pipe not found
-                    continue;   // Skip line
-                }
-
-                i[0] = '\0';
-                xastir_snprintf(state,sizeof(state),"%s",j+1);
-                clean_string(state);
-
 //NOTE:  It'd be nice to take the part after the comma and put it before the rest
 // of the text someday, i.e. "Cassidy, Lake".
 
                 // Find end of Feature Name field
-                j = index(i+1, '|');
+                i = index(++j, '|');
 
-                if (j == NULL) {    // Pipe not found
+                if (i == NULL) {    // Pipe not found
                     continue;   // Skip line
                 }
 
-                j[0] = '\0';
-                xastir_snprintf(name,sizeof(name),"%s",i+1);
+                i[0] = '\0';
+                xastir_snprintf(name,sizeof(name),"%s",j);
                 clean_string(name);
 
                 // Find end of Feature Type field
-                i = index(j+1, '|');
-
-                if (i == NULL) {    // Pipe not found
-                    continue;   // Skip line
-                }
-
-                i[0] = '\0';
-                xastir_snprintf(type,sizeof(type),"%s",j+1);
-                clean_string(type);
-
-                // Find end of County Name field
-                j = index(i+1, '|');
+                j = index(++i, '|');
 
                 if (j == NULL) {    // Pipe not found
                     continue;   // Skip line
                 }
 
                 j[0] = '\0';
-                xastir_snprintf(county,sizeof(county),"%s",i+1);
-                clean_string(county);
+                xastir_snprintf(type,sizeof(type),"%s",i);
+                clean_string(type);
 
-                // Find end of State Number Code field
-                i = index(j+1, '|');
+                // Find end of State field
+                i = index(++j,'|');
 
                 if (i == NULL) {    // Pipe not found
                     continue;   // Skip line
                 }
 
                 i[0] = '\0';
+                xastir_snprintf(state,sizeof(state),"%s",j);
+                clean_string(state);
+
+                // Find end of State Number Code field
+                j = index(++i, '|');
+
+                if (j == NULL) {    // Pipe not found
+                    continue;   // Skip line
+                }
+
+                j[0] = '\0';
+
+                // Find end of County Name field
+                i = index(++j, '|');
+
+                if (i == NULL) {    // Pipe not found
+                    continue;   // Skip line
+                }
+
+                i[0] = '\0';
+                xastir_snprintf(county,sizeof(county),"%s",j);
+                clean_string(county);
 
                 // Find end of County Number Code field
-                j = index(i+1, '|');
+                j = index(++i, '|');
 
                 if (j == NULL) {    // Pipe not found
                     continue;   // Skip line
@@ -1071,30 +1160,30 @@ int gnis_locate_place( Widget w,
                 j[0] = '\0';
 
                 // Find end of Primary Latitude field (DDMMSSN)
-                i = index(j+1, '|');
+                i = index(++j, '|');
 
                 if (i == NULL) {    // Pipe not found
                     continue;   // Skip line
                 }
 
                 i[0] = '\0';
-                xastir_snprintf(latitude,sizeof(latitude),"%s",j+1);
+                xastir_snprintf(latitude,sizeof(latitude),"%s",j);
                 clean_string(latitude);
 
                 // Find end of Primary Longitude field (DDDMMSSW)
-                j = index(i+1, '|');
+                j = index(++i, '|');
 
                 if (j == NULL) {    // Pipe not found
                     continue;   // Skip line
                 }
 
                 j[0] = '\0';
-                xastir_snprintf(longitude,sizeof(longitude),"%s",i+1);
+                xastir_snprintf(longitude,sizeof(longitude),"%s",i);
                 clean_string(longitude);
 
                 // Find end of Primary Latitude field (decimal
                 // degrees)
-                i = index(j+1, '|');
+                i = index(++j, '|');
 
                 if (i == NULL) {    // Pipe not found
                     continue;   // Skip line
@@ -1104,7 +1193,7 @@ int gnis_locate_place( Widget w,
 
                 // Find end of Primary Longitude field (decimal
                 // degrees)
-                j = index(i+1, '|');
+                j = index(++i, '|');
 
                 if (j == NULL) {    // Pipe not found
                     continue;   // Skip line
@@ -1113,7 +1202,7 @@ int gnis_locate_place( Widget w,
                 j[0] = '\0';
 
                 // Find end of Source Latitude field (DMS)
-                i = index(j+1, '|');
+                i = index(++j, '|');
 
                 if (i == NULL) {    // Pipe not found
                     continue;   // Skip line
@@ -1122,7 +1211,7 @@ int gnis_locate_place( Widget w,
                 i[0] = '\0';
 
                 // Find end of Source Longitude (DMS)
-                j = index(i+1, '|');
+                j = index(++i, '|');
 
                 if (j == NULL) {    // Pipe not found
                     continue;   // Skip line
@@ -1132,7 +1221,7 @@ int gnis_locate_place( Widget w,
 
                 // Find end of Source Latitude field (decimal
                 // degrees)
-                i = index(j+1, '|');
+                i = index(++j, '|');
 
                 if (i == NULL) {    // Pipe not found
                     continue;   // Skip line
@@ -1142,7 +1231,7 @@ int gnis_locate_place( Widget w,
 
                 // Find end of Source Longitude field (decimal
                 // degrees)
-                j = index(i+1, '|');
+                j = index(++i, '|');
 
                 if (j == NULL) {    // Pipe not found
                     continue;   // Skip line
@@ -1150,8 +1239,8 @@ int gnis_locate_place( Widget w,
 
                 j[0] = '\0';
 
-                // Find end of Elevation field
-                i = index(j+1, '|');
+                // Find end of Estimated Population field
+                i = index(++j, '|');
 
                 if (i == NULL) {    // Pipe not found
                     continue;   // Skip line
@@ -1159,19 +1248,19 @@ int gnis_locate_place( Widget w,
 
                 i[0] = '\0';
 
-                // Find end of Estimated Population field
-                j = index(i+1, '|');
+                xastir_snprintf(population,sizeof(population),"%s",j);
+                clean_string(population);
+
+                // Find end of Quad field
+                j = index(++i, '|');
 
                 if (j == NULL) {    // Pipe not found
                     continue;   // Skip line
                 }
 
                 j[0] = '\0';
-                xastir_snprintf(population,sizeof(population),"%s",i+1);
-                clean_string(population);
 
-                // Snag Cell Name field (Quad name, last field)
-                xastir_snprintf(quad,sizeof(quad),"%s",j+1);
+                xastir_snprintf(quad,sizeof(quad),"%s",i);
                 clean_string(quad);
 
                 // If "Match Case" togglebutton is not set, convert
@@ -1236,6 +1325,8 @@ int gnis_locate_place( Widget w,
 // "Found It!"
 //popup_message_always( langcode("POPEM00029"), name );
 
+		    if (strlen(latitude) < 7) 
+			continue;	// We really don't have any latitude here.
                     lat_dd[0] = latitude[0];
                     lat_dd[1] = latitude[1];
                     lat_dd[2] = '\0';
@@ -1251,6 +1342,8 @@ int gnis_locate_place( Widget w,
                     lat_dir[0] = latitude[6];
                     lat_dir[1] = '\0';
 
+		    if (strlen(longitude) < 8) 
+			continue;	// We really don't have any longitude here.
                     long_dd[0] = longitude[0];
                     long_dd[1] = longitude[1];
                     long_dd[2] = longitude[2];
