@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_tif.c,v 1.24 2005/07/28 20:41:19 we7u Exp $
+ * $Id: map_tif.c,v 1.47 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,7 +24,10 @@
 
 
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -37,7 +40,9 @@
 #include <errno.h>
 
 // Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
 
 #include <dirent.h>
 #include <netinet/in.h>
@@ -78,11 +83,22 @@ extern int npoints;		/* tsk tsk tsk -- globals */
 extern int mag;
 
 #ifdef HAVE_LIBGEOTIFF
-//#include "cpl_csv.h"
+
+
 #include "xtiffio.h"
-#include "geotiffio.h"
+//#include "geotiffio.h"
+
 #include "geo_normalize.h"
-#include "projects.h"
+
+
+// Needed for GTIFProj4FromLatLong() replacement below
+#ifdef HAVE_LIBGDAL
+#  include "proj_api.h"
+#endif  // HAVE_LIBGDAL
+
+
+// Must be last include file
+#include "leak_detection.h"
 
 
 
@@ -330,6 +346,90 @@ int read_fgd_file ( char* tif_filename,
 
 
 
+#ifndef HAVE_LIBGDAL
+#  define my_GTIFProj4FromLatLong GTIFProj4FromLatLong
+#else
+
+// If we have libgdal included then the GTIFProj4FromLatLong()
+// functions may not work.  In this case we use the function below
+// instead.
+
+
+// This next function was borrowed from the libgeotiff project.  We
+// have permission from Frank Warmerdam to use any code from
+// geotiff_proj4.c and relicense it under GPL, per private message
+// of 10/14/2005.  Frank wrote that module.
+//
+// When we used the original function from libgeotiff we'd lose topo
+// map capability if we linked in libgdal.  We avoid that problem by
+// having a local copy of the function here with a different name so
+// that libgdal can't try to override it.
+//
+//***********************************************************************
+//                        my_GTIFProj4FromLatLong()
+//
+//      Convert lat/long values to projected coordinate for a
+//      particular definition.
+//***********************************************************************
+
+int my_GTIFProj4FromLatLong( GTIFDefn * psDefn,
+                             int nPoints,
+                             double *padfX,
+                             double *padfY ) {
+
+    char    *pszProjection;
+    projPJ      psPJ;
+    int     i;
+
+// --------------------------------------------------------------------
+//  Get a projection definition.
+// --------------------------------------------------------------------
+    pszProjection = GTIFGetProj4Defn( psDefn );
+
+    if( pszProjection == NULL )
+        return FALSE;
+
+//fprintf(stderr,"pszProjection:%s\n", pszProjection);
+// Returned this:
+// pszProjection:+proj=utm +zone=10 +ellps=clrk66 +units=m 
+
+
+// --------------------------------------------------------------------
+//      Initialize the projection
+// --------------------------------------------------------------------
+
+    psPJ = pj_init_plus(pszProjection);
+
+    if( psPJ == NULL ) {
+        return FALSE;
+    }
+
+// --------------------------------------------------------------------
+//      Process each of the points.
+// --------------------------------------------------------------------
+
+    for( i = 0; i < nPoints; i++ ) {
+        projUV  sUV;
+
+        sUV.u = padfX[i] * DEG_TO_RAD;
+        sUV.v = padfY[i] * DEG_TO_RAD;
+
+        sUV = pj_fwd( sUV, psPJ );
+
+        padfX[i] = sUV.u;
+        padfY[i] = sUV.v;
+    }
+
+    pj_free( psPJ );
+
+    return TRUE;
+}
+#endif  // HAVE_LIBGDAL
+
+
+
+
+
 // Check the "GTIFProj4*" functions to see if we can use
 // datum.c instead for any of them.  These calls are all in
 // draw_geotiff_image_map().  It looks like they deal with
@@ -706,18 +806,20 @@ void draw_geotiff_image_map (Widget w,
      * or keys that I need individually instead.  I
      * need "defn" for the GTIFProj4ToLatLong calls though.
      */
-    if (GTIFGetDefn (gtif, &defn))
-    {
+    if (GTIFGetDefn (gtif, &defn)) {
         if (debug_level & 16)
             GTIFPrintDefn (&defn, stdout);
     }
+    else {
+        fprintf(stderr,"GTIFGetDefn failed\n");
+    }
 
     proj_is_latlong=FALSE;
-    if( !GTIFKeyGet(gtif,ProjectedCSTypeGeoKey, &PCS,0,1))
-      {
-          //fprintf(stderr,"Warning: no PCS in geotiff file %s, assuming map is in lat/lon!\n", filenm);
-	proj_is_latlong=TRUE;
-      }
+
+    if( !GTIFKeyGet(gtif,ProjectedCSTypeGeoKey, &PCS,0,1)) {
+        // fprintf(stderr,"Warning: no PCS in geotiff file %s, assuming map is in lat/lon!\n", filenm);
+        proj_is_latlong=TRUE;
+    }
 
     /* Fetch a few TIFF fields for this image */
     if ( !TIFFGetField (tif, TIFFTAG_IMAGEWIDTH, &width) ) {
@@ -790,6 +892,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
+
         if ( proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )   // Do all 4 of these in one call?
         {
             if (debug_level & 16) {
@@ -798,6 +901,12 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"%f  %f\n", xxx, yyy);
             }
         }
+        else {
+            if (!proj_is_latlong) {
+                fprintf(stderr,"Failed GTIFProj4ToLatLong() call\n");
+            }
+        }
+
         f_NW_x_bounding = (float)xxx;
         f_NW_y_bounding = (float)yyy;
 
@@ -812,6 +921,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
+
         if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
@@ -820,6 +930,12 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"%f  %f\n", xxx, yyy);
             }
         }
+        else {
+            if (!proj_is_latlong) {
+                fprintf(stderr,"Failed GTIFProj4ToLatLong() call\n");
+            }
+        }
+
         f_NE_x_bounding = (float)xxx;
         f_NE_y_bounding = (float)yyy;
 
@@ -833,6 +949,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
+
         if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
@@ -841,6 +958,12 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"%f  %f\n", xxx, yyy);
             }
         }
+        else {
+            if (!proj_is_latlong) {
+                fprintf(stderr,"Failed GTIFProj4ToLatLong() call\n");
+            }
+        }
+
         f_SW_x_bounding = (float)xxx;
         f_SW_y_bounding = (float)yyy;
 
@@ -854,6 +977,7 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"(%11.3f,%11.3f)\n", xxx, yyy );
             }
         }
+
         if (  proj_is_latlong || GTIFProj4ToLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16) {
@@ -862,6 +986,12 @@ void draw_geotiff_image_map (Widget w,
                 fprintf(stderr,"%f  %f\n", xxx, yyy);
             }
         }
+        else {
+            if (!proj_is_latlong) {
+                fprintf(stderr,"Failed GTIFProj4ToLatLong() call\n");
+            }
+        }
+
         f_SE_x_bounding = (float)xxx;
         f_SE_y_bounding = (float)yyy;
 
@@ -1153,7 +1283,16 @@ void draw_geotiff_image_map (Widget w,
         GTIFFree (gtif);
         XTIFFClose (tif);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1318,7 +1457,16 @@ Samples Per Pixel: 1
             GTIFFree (gtif);
             XTIFFClose (tif);
             // Update to screen
-            (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+            (void)XCopyArea(XtDisplay(da),
+                pixmap,
+                XtWindow(da),
+                gc,
+                0,
+                0,
+                (unsigned int)screen_width,
+                (unsigned int)screen_height,
+                0,
+                0);
             return;
         }
 
@@ -1331,14 +1479,14 @@ Samples Per Pixel: 1
         yyy = (double)f_NW_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )     // Do all 4 in one call?
-        {
+        if (  proj_is_latlong || my_GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) ) {     // Do all 4 in one call?
+
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
 
             /* Convert from PCS coordinates to image pixel coordinates */
-            if ( GTIFPCSToImage( gtif, &xxx, &yyy ) )           // Do all 4 in one call?
-            {
+            if ( GTIFPCSToImage( gtif, &xxx, &yyy ) ) { // Do all 4 in one call?
+
                 if (debug_level & 16)
                     fprintf(stderr,"X/Y Pixels: %f, %f\n", xxx, yyy);
 
@@ -1409,7 +1557,7 @@ Samples Per Pixel: 1
         yyy = (double)f_NE_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || my_GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1487,7 +1635,7 @@ Samples Per Pixel: 1
         yyy = (double)f_SW_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || my_GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1565,7 +1713,7 @@ Samples Per Pixel: 1
         yyy = (double)f_SE_y_bounding;
 
         /* Convert lat/long to projected coordinates */
-        if (  proj_is_latlong || GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
+        if (  proj_is_latlong || my_GTIFProj4FromLatLong( &defn, 1, &xxx, &yyy ) )
         {
             if (debug_level & 16)
                 fprintf(stderr,"%11.3f,%11.3f\n", xxx, yyy);
@@ -1758,11 +1906,14 @@ right_crop = width - 1;
                 my_colors[l].green = (uint16)(green_orig[l] * raster_map_intensity);
                 my_colors[l].blue  =  (uint16)(blue_orig[l] * raster_map_intensity);
 
-                if (visual_type == NOT_TRUE_NOR_DIRECT)
+                if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                    XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                     XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-                else
+                }
+                else {
                     pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                     &my_colors[l].pixel);
+                }
             }
             break;
         case PHOTOMETRIC_MINISBLACK:
@@ -1772,11 +1923,14 @@ right_crop = width - 1;
                 my_colors[l].red = my_colors[l].green = my_colors[l].blue =
                     (uint16)(v * raster_map_intensity) << 8;
 
-                if (visual_type == NOT_TRUE_NOR_DIRECT)
+                if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                    XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                     XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-                else
+                }
+                else {
                     pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                     &my_colors[l].pixel);
+                }
             }
             break;
         case PHOTOMETRIC_MINISWHITE:
@@ -1786,11 +1940,14 @@ right_crop = width - 1;
                 my_colors[l].red = my_colors[l].green = my_colors[l].blue =
                   (uint16)(v * raster_map_intensity) << 8;
 
-                if (visual_type == NOT_TRUE_NOR_DIRECT)
+                if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                    XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                     XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-                else
+                }
+                else {
                     pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                     &my_colors[l].pixel);
+                }
             }
             break;
         }
@@ -2222,19 +2379,19 @@ right_crop = width - 1;
 
     // Find edges of current viewport in Xastir coordinates
     //
-    view_min_x = x_long_offset;                         /*   left edge of view */
+    view_min_x = NW_corner_longitude;  // left edge of view
     if (view_min_x > 129600000l)
         view_min_x = 0;
 
-    view_max_x = x_long_offset + (screen_width * scale_x); /*  right edge of view */
+    view_max_x = SE_corner_longitude; // right edge of view 
     if (view_max_x > 129600000l)
         view_max_x = 129600000l;
 
-    view_min_y = y_lat_offset;                          /*    top edge of view */
+    view_min_y = NW_corner_latitude;   // top edge of view
     if (view_min_y > 64800000l)
         view_min_y = 0;
 
-    view_max_y = y_lat_offset + (screen_height * scale_y); /* bottom edge of view */
+    view_max_y = SE_corner_latitude;  // bottom edge of view
     if (view_max_y > 64800000l)
         view_max_y = 64800000l;
 
@@ -2386,6 +2543,17 @@ right_crop = width - 1;
         int skip = 0;
 
 
+        HandlePendingEvents(app_context);
+        if (interrupt_drawing_now) {
+            if (imageMemory)
+                free(imageMemory);
+            GTIFFree (gtif);
+            XTIFFClose (tif);
+            // Update to screen
+            (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+            return;
+        }
+
         // Our offset from the top row of the map neatline
         // (kind of... ignoring rotation anyway).
         row_offset = row - top_crop;
@@ -2559,15 +2727,15 @@ if (current_right >= width)
                     // Here are the corners of our viewport, using the Xastir
                     // coordinate system.  Notice that Y is upside down:
                     // 
-                    // left edge of view = x_long_offset
-                    // right edge of view = x_long_offset + (screen_width  * scale_x)
-                    // top edge of view =  y_lat_offset
-                    // bottom edge of view =  y_lat_offset + (screen_height * scale_y)
+                    // left edge of view = NW_corner_longitude
+                    // right edge of view = SE_corner_longitude
+                    // top edge of view =  NW_corner_latitude
+                    // bottom edge of view =  SE_corner_latitude
 
 
                     // Compute the screen position of the pixel and scale it
-                    sxx = (xastir_current_x - x_long_offset) / scale_x;
-                    syy = (xastir_total_y   - y_lat_offset ) / scale_y;
+                    sxx = (xastir_current_x - NW_corner_longitude) / scale_x;
+                    syy = (xastir_total_y   - NW_corner_latitude ) / scale_y;
 
 
 
