@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_tiger.c,v 1.49 2006/04/24 19:21:42 we7u Exp $
+ * $Id: map_tiger.c,v 1.68 2012/09/23 16:19:22 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2006  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -65,6 +65,7 @@
 #include "xastir.h"
 #include "maps.h"
 #include "alert.h"
+#include "fetch_remote.h"
 #include "util.h"
 #include "main.h"
 #include "datum.h"
@@ -77,7 +78,7 @@
 
 #define CHECKMALLOC(m)  if (!m) { fprintf(stderr, "***** Malloc Failed *****\n"); exit(0); }
 
-#ifdef HAVE_IMAGEMAGICK
+#ifdef HAVE_MAGICK
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -89,6 +90,15 @@
 # endif // HAVE_SYS_TIME_H
 #endif  // TIME_WITH_SYS_TIME
 #undef RETSIGTYPE
+// TVR: "stupid ImageMagick"
+// The problem is that magick/api.h includes Magick's config.h file, and that
+// pulls in all the same autoconf-generated defines that we use.
+// plays those games below, but I don't think in the end that they actually 
+// make usable macros with our own data in them.
+// Fortunately, we don't need them, so I'll just undef the ones that are
+// causing problems today.  See main.c for fixes that preserve our values.
+#undef PACKAGE
+#undef VERSION
 /* JMT - stupid ImageMagick */
 #define XASTIR_PACKAGE_BUGREPORT PACKAGE_BUGREPORT
 #undef PACKAGE_BUGREPORT
@@ -100,7 +110,12 @@
 #undef PACKAGE_TARNAME
 #define XASTIR_PACKAGE_VERSION PACKAGE_VERSION
 #undef PACKAGE_VERSION
+#ifdef HAVE_GRAPHICSMAGICK
+/*#include <GraphicsMagick/magick/api.h>*/
 #include <magick/api.h>
+#else   // HAVE_GRAPHICSMAGICK
+#include <magick/api.h>
+#endif  // HAVE_GRAPHICSMAGICK 
 #undef PACKAGE_BUGREPORT
 #define PACKAGE_BUGREPORT XASTIR_PACKAGE_BUGREPORT
 #undef XASTIR_PACKAGE_BUGREPORT
@@ -116,7 +131,7 @@
 #undef PACKAGE_VERSION
 #define PACKAGE_VERSION XASTIR_PACKAGE_VERSION
 #undef XASTIR_PACKAGE_VERSION
-#endif // HAVE_IMAGEMAGICK
+#endif // HAVE_MAGICK
 
 // Must be last include file
 #include "leak_detection.h"
@@ -131,7 +146,7 @@ extern int mag;
 
 
 void get_tiger_local_file(char * local_filename, char * fileimg){
-#ifdef HAVE_IMAGEMAGICK
+#ifdef HAVE_MAGICK
 
 //  char local_filename[MAX_FILENAME];
 
@@ -142,8 +157,10 @@ void get_tiger_local_file(char * local_filename, char * fileimg){
 
 #ifdef USE_MAP_CACHE 
     int map_cache_return = 1; // Default = cache miss
+    char *cache_file_id;
 #endif  // USE_MAP_CACHE
 
+    char temp_file_path[MAX_VALUE];
 
     if (debug_level & 512) {
         query_start_time=time(&query_start_time); 
@@ -177,12 +194,14 @@ clear_dangerous();
     if (map_cache_return != 0 ) {
 
 set_dangerous("map_tiger: map_cache_fileid");
+        cache_file_id = map_cache_fileid();
         xastir_snprintf(local_filename,
             MAX_FILENAME,           // hardcoded to avoid sizeof()
             "%s/map_%s.%s",
-            get_user_base_dir("map_cache"),
-            map_cache_fileid(),
+            get_user_base_dir("map_cache", temp_file_path, sizeof(temp_file_path)),
+            cache_file_id,
             "gif");
+        free(cache_file_id);
 clear_dangerous();
 
 #else   // USE_MAP_CACHE
@@ -190,7 +209,7 @@ clear_dangerous();
     xastir_snprintf(local_filename,
         MAX_FILENAME,               // hardcoded to avoid sizeof()
         "%s/map.%s",
-         get_user_base_dir("tmp"),
+         get_user_base_dir("tmp", temp_file_path, sizeof(temp_file_path)),
         "gif");
 
 #endif  // USE_MAP_CACHE
@@ -230,7 +249,7 @@ clear_dangerous();
     // Set permissions on the file so that any user can overwrite it.
     chmod(local_filename, 0666);
 
-#endif  //HAVE_IMAGEMAGICK
+#endif  //HAVE_MAGICK
 
 } // end get_tiger_local_file
 
@@ -242,7 +261,7 @@ clear_dangerous();
  * draw_tiger_map()
  * N0VH
  **********************************************************/
-#ifdef HAVE_IMAGEMAGICK
+#ifdef HAVE_MAGICK
 void draw_tiger_map (Widget w,
         char *filenm,
         int destination_pixmap,
@@ -365,21 +384,21 @@ void draw_tiger_map (Widget w,
     //
     tp[0].img_x = 0;                // Pixel Coordinates
     tp[0].img_y = 0;                // Pixel Coordinates
-    tp[0].x_long = x_long_offset;   // Xastir Coordinates
-    tp[0].y_lat  = y_lat_offset;    // Xastir Coordinates
+    tp[0].x_long = NW_corner_longitude;   // Xastir Coordinates
+    tp[0].y_lat  = NW_corner_latitude;    // Xastir Coordinates
 
     // Tiepoint for lower right screen corner
     //
     tp[1].img_x =  screen_width - 1; // Pixel Coordinates
     tp[1].img_y = screen_height - 1; // Pixel Coordinates 
 
-    tp[1].x_long = x_long_offset + (( screen_width) * scale_x); // Xastir Coordinates
-    tp[1].y_lat  =  y_lat_offset + ((screen_height) * scale_y); // Xastir Coordinates
+    tp[1].x_long = SE_corner_longitude; // Xastir Coordinates
+    tp[1].y_lat  =  SE_corner_latitude; // Xastir Coordinates
 
-    left = (double)((x_long_offset - 64800000l )/360000.0);   // Lat/long Coordinates
-    top = (double)(-((y_lat_offset - 32400000l )/360000.0));  // Lat/long Coordinates
-    right = (double)(((x_long_offset + ((screen_width) * scale_x) ) - 64800000l)/360000.0);//Lat/long Coordinates
-    bottom = (double)(-(((y_lat_offset + ((screen_height) * scale_y) ) - 32400000l)/360000.0));//Lat/long Coordinates
+    left = (double)((NW_corner_longitude - 64800000l )/360000.0);   // Lat/long Coordinates
+    top = (double)(-((NW_corner_latitude - 32400000l )/360000.0));  // Lat/long Coordinates
+    right = (double)((SE_corner_longitude - 64800000l)/360000.0);//Lat/long Coordinates
+    bottom = (double)(-((SE_corner_latitude - 32400000l)/360000.0));//Lat/long Coordinates
 
     map_width = right - left;   // Lat/long Coordinates
     map_height = top - bottom;  // Lat/long Coordinates
@@ -398,81 +417,81 @@ void draw_tiger_map (Widget w,
     xastir_snprintf(tigertmp, sizeof(tigertmp), "http://tiger.census.gov/cgi-bin/mapper/map.gif?");
 
     if (tiger_show_grid)
-        strncat(tigertmp, "&on=GRID", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=GRID", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=GRID", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=GRID", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_counties)
-        strncat(tigertmp, "&on=counties", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=counties", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=counties", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=counties", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_cities)
-        strncat(tigertmp, "&on=CITIES", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=CITIES", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=CITIES", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=CITIES", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_places)
-        strncat(tigertmp, "&on=places", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=places", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=places", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=places", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_majroads)
-        strncat(tigertmp, "&on=majroads", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=majroads", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=majroads", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=majroads", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_streets)
-        strncat(tigertmp, "&on=streets", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=streets", sizeof(tigertmp) - 1 - strlen(tigertmp));
     // Don't turn streets off since this will automagically show up as you zoom in.
 
     if (tiger_show_railroad)
-        strncat(tigertmp, "&on=railroad", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=railroad", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=railroad", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=railroad", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_states)
-        strncat(tigertmp, "&on=states", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=states", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=states", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=states", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_interstate)
-        strncat(tigertmp, "&on=interstate", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=interstate", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=interstate", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=interstate", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_ushwy)
-        strncat(tigertmp, "&on=ushwy", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=ushwy", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=ushwy", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=ushwy", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_statehwy)
-        strncat(tigertmp, "&on=statehwy", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=statehwy", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=statehwy", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=statehwy", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_water)
-        strncat(tigertmp, "&on=water", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=water", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=water", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=water", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_lakes)
-        strncat(tigertmp, "&on=shorelin", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=shorelin", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=shorelin", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=shorelin", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     if (tiger_show_misc)
-        strncat(tigertmp, "&on=miscell", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&on=miscell", sizeof(tigertmp) - 1 - strlen(tigertmp));
     else
-        strncat(tigertmp, "&off=miscell", sizeof(tigertmp) - strlen(tigertmp));
+        strncat(tigertmp, "&off=miscell", sizeof(tigertmp) - 1 - strlen(tigertmp));
 
     xastir_snprintf(tmpstr, sizeof(tmpstr), "&lat=%f\046lon=%f\046", lat_center, long_center);    
-    strncat (tigertmp, tmpstr, sizeof(tigertmp) - strlen(tigertmp));
+    strncat (tigertmp, tmpstr, sizeof(tigertmp) - 1 - strlen(tigertmp));
     xastir_snprintf(tmpstr, sizeof(tmpstr), "wid=%f\046ht=%f\046", map_width, map_height);
-    strncat (tigertmp, tmpstr, sizeof(tigertmp) - strlen(tigertmp));
+    strncat (tigertmp, tmpstr, sizeof(tigertmp) - 1 - strlen(tigertmp));
     xastir_snprintf(tmpstr, sizeof(tmpstr), "iwd=%i\046iht=%i", tp[1].img_x + 1, tp[1].img_y + 1);
-    strncat (tigertmp, tmpstr, sizeof(tigertmp) - strlen(tigertmp));
-    xastir_snprintf(fileimg, sizeof(fileimg), tigertmp);
+    strncat (tigertmp, tmpstr, sizeof(tigertmp) - 1 - strlen(tigertmp));
+    xastir_snprintf(fileimg, sizeof(fileimg), "%s", tigertmp);
 
     if (debug_level & 512) {
           fprintf(stderr,"left side is %f\n", left);
@@ -550,6 +569,20 @@ void draw_tiger_map (Widget w,
     if (f == NULL) {
         if (debug_level & 512)
             fprintf(stderr,"File could not be read\n");
+
+#ifdef USE_MAP_CACHE
+
+        // clear from cache if bad    
+        if (map_cache_del(fileimg)) {
+            if (debug_level & 512) {
+                fprintf(stderr,"Couldn't delete unreadable map from cache\n");
+            }
+        }
+#endif
+         
+        if (image_info)
+            DestroyImageInfo(image_info);
+	DestroyExceptionInfo(&exception);
         return;
     }
     (void)fclose (f);
@@ -560,6 +593,19 @@ void draw_tiger_map (Widget w,
     if (image == (Image *) NULL) {
         MagickWarning(exception.severity, exception.reason, exception.description);
         //fprintf(stderr,"MagickWarning\n");
+
+#ifdef USE_MAP_CACHE
+        // clear from cache if bad    
+        if (map_cache_del(fileimg)) {
+            if (debug_level & 512) {
+                fprintf(stderr,"Couldn't delete map from cache\n");
+            }
+        }
+#endif
+
+        if (image_info)
+            DestroyImageInfo(image_info);
+	DestroyExceptionInfo(&exception);
         return;
     }
 
@@ -573,6 +619,7 @@ void draw_tiger_map (Widget w,
             DestroyImage(image);
         if (image_info)
             DestroyImageInfo(image_info);
+	DestroyExceptionInfo(&exception);
         return;
     }
 
@@ -625,6 +672,7 @@ void draw_tiger_map (Widget w,
             DestroyImage(image);
         if (image_info)
             DestroyImageInfo(image_info);
+	DestroyExceptionInfo(&exception);
         return;
     }
 
@@ -636,6 +684,7 @@ void draw_tiger_map (Widget w,
             DestroyImage(image);
         if (image_info)
             DestroyImageInfo(image_info);
+	DestroyExceptionInfo(&exception);
         return;
     }
 
@@ -691,10 +740,10 @@ void draw_tiger_map (Widget w,
     * Here are the corners of our viewport, using the Xastir
     * coordinate system.  Notice that Y is upside down:
     *
-    *   left edge of view = x_long_offset
-    *  right edge of view = x_long_offset + (screen_width  * scale_x)
-    *    top edge of view =  y_lat_offset
-    * bottom edge of view =  y_lat_offset + (screen_height * scale_y)
+    *   left edge of view = NW_corner_longitude
+    *  right edge of view = SE_corner_longitude
+    *    top edge of view =  NW_corner_latitude
+    * bottom edge of view =  SE_corner_latitude
     *
     * The corners of our map will soon be (after translating the
     * tiepoints to the corners if they're not already there):
@@ -705,8 +754,8 @@ void draw_tiger_map (Widget w,
     * bottom edge of map = tp[1].y_lat
     *
     */
-    map_c_L = tp[0].x_long - x_long_offset;     // map left coordinate
-    map_c_T = tp[0].y_lat  - y_lat_offset;      // map top  coordinate
+    map_c_L = tp[0].x_long - NW_corner_longitude;     // map left coordinate
+    map_c_T = tp[0].y_lat  - NW_corner_latitude;      // map top  coordinate
 
     tp_c_dx = (long)(tp[1].x_long - tp[0].x_long);//  Width between tiepoints
     tp_c_dy = (long)(tp[1].y_lat  - tp[0].y_lat); // Height between tiepoints
@@ -729,14 +778,14 @@ void draw_tiger_map (Widget w,
     // calculate top left map corner from tiepoints
     if (tp[0].img_x != 0) {
         tp[0].x_long -= (tp[0].img_x * map_c_dx);   // map left edge longitude
-        map_c_L = tp[0].x_long - x_long_offset;     // delta ??
+        map_c_L = tp[0].x_long - NW_corner_longitude;     // delta ??
         tp[0].img_x = 0;
         if (debug_level & 512)
             fprintf(stderr,"Translated tiepoint_0 x: %d\t%lu\n", tp[0].img_x, tp[0].x_long);
     }
     if (tp[0].img_y != 0) {
         tp[0].y_lat -= (tp[0].img_y * map_c_dy);    // map top edge latitude
-        map_c_T = tp[0].y_lat - y_lat_offset;
+        map_c_T = tp[0].y_lat - NW_corner_latitude;
         tp[0].img_y = 0;
         if (debug_level & 512)
             fprintf(stderr,"Translated tiepoint_0 y: %d\t%lu\n", tp[0].img_y, tp[0].y_lat);
@@ -787,13 +836,13 @@ void draw_tiger_map (Widget w,
 
     map_c_xc  = (tp[0].x_long + tp[1].x_long) / 2;  // hor center of map as reference
     map_x_ctr = (long)(width  / 2 +0.499);
-    scr_x_mc  = (map_c_xc - x_long_offset) / scale_x; // screen coordinates of map center
+    scr_x_mc  = (map_c_xc - NW_corner_longitude) / scale_x; // screen coordinates of map center
 
     // calculate map pixel range in y direction that falls into screen area
     c_y_max = 0ul;
     map_y_min = map_y_max = 0l;
     for (map_y_0 = 0, c_y = tp[0].y_lat; map_y_0 < (long)height; map_y_0++, c_y += map_c_dy) {
-        scr_y = (c_y - y_lat_offset) / scale_y;   // current screen position
+        scr_y = (c_y - NW_corner_latitude) / scale_y;   // current screen position
         if (scr_y > 0) {
             if (scr_y < screen_height) {
                 map_y_max = map_y_0;          // update last map pixel in y
@@ -808,7 +857,7 @@ void draw_tiger_map (Widget w,
 
         map_x_min = map_x_max = 0l;
         for (map_x = 0, c_x = tp[0].x_long; map_x < (long)width; map_x++, c_x += map_c_dx) {
-            scr_x = (c_x - x_long_offset)/ scale_x;  // current screen position
+            scr_x = (c_x - NW_corner_longitude)/ scale_x;  // current screen position
             if (scr_x > 0) {
                 if (scr_x < screen_width)
                     map_x_max = map_x;          // update last map pixel in x
@@ -821,7 +870,7 @@ void draw_tiger_map (Widget w,
         c_x_min = (unsigned long)(tp[0].x_long + map_x_min * map_c_dx);   // left map inside screen coordinate
 
     scr_yp = -1;
-    scr_c_xr = x_long_offset + screen_width * scale_x;
+    scr_c_xr = SE_corner_longitude;
     c_dx = map_c_dx;                            // map pixel width
     scale_xa = scale_x0;                        // the compiler likes it ;-)
 
@@ -851,10 +900,11 @@ void draw_tiger_map (Widget w,
                 (unsigned int)screen_height,
                 0,
                 0);
+            DestroyExceptionInfo(&exception);
             return;
         }
 
-        scr_y = (c_y - y_lat_offset) / scale_y;
+        scr_y = (c_y - NW_corner_latitude) / scale_y;
         if (scr_y != scr_yp) {                  // don't do a row twice
             scr_yp = scr_y;                     // remember as previous y
             scr_xp = -1;
@@ -862,7 +912,7 @@ void draw_tiger_map (Widget w,
             map_act = 0;
             scale_x_nm = calc_dscale_x(0,(long)c_y) / 1852.0;  // nm per Xastir coordinate
             for (map_x = map_x_min, c_x = (double)c_x_min; map_x <= map_x_max; map_x++, c_x += c_dx) {
-                scr_x = (c_x - x_long_offset) / scale_x;
+                scr_x = (c_x - NW_corner_longitude) / scale_x;
                 if (scr_x != scr_xp) {      // don't do a pixel twice
                     scr_xp = scr_x;         // remember as previous x
                     map_y = map_y_0;
@@ -877,9 +927,26 @@ void draw_tiger_map (Widget w,
                             XSetForeground(XtDisplay(w), gc, my_colors[index_pack[l]].pixel);
                         }
                         else {
-                            pack_pixel_bits(pixel_pack[l].red * raster_map_intensity,
-                                            pixel_pack[l].green * raster_map_intensity,
-                                            pixel_pack[l].blue * raster_map_intensity,
+                            // It is not safe to assume that the red/green/blue
+                            // elements of pixel_pack of type Quantum are the
+                            // same as the red/green/blue of an XColor!
+                            if (QuantumDepth==16) {
+                                my_colors[0].red=pixel_pack[l].red;
+                                my_colors[0].green=pixel_pack[l].green;
+                                my_colors[0].blue=pixel_pack[l].blue;
+                            }
+                            else { // QuantumDepth=8
+                                // shift the bits of the 8-bit quantity so that
+                                // they become the high bigs of my_colors.*
+                                my_colors[0].red=pixel_pack[l].red<<8;
+                                my_colors[0].green=pixel_pack[l].green<<8;
+                                my_colors[0].blue=pixel_pack[l].blue<<8;
+                            }
+                            // NOW my_colors has the right r,g,b range for
+                            // pack_pixel_bits
+                            pack_pixel_bits(my_colors[0].red * raster_map_intensity,
+                                            my_colors[0].green * raster_map_intensity,
+                                            my_colors[0].blue * raster_map_intensity,
                                             &my_colors[0].pixel);
                             XSetForeground(XtDisplay(w), gc, my_colors[0].pixel);
                         }
@@ -896,6 +963,7 @@ void draw_tiger_map (Widget w,
        DestroyImage(image);
     if (image_info)
        DestroyImageInfo(image_info);
+    DestroyExceptionInfo(&exception);
 }
-#endif //HAVE_IMAGEMAGICK
+#endif //HAVE_MAGICK
 ///////////////////////////////////////////// End of Tigermap code ///////////////////////////////////////
