@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: maps.h,v 1.60 2006/02/28 19:30:00 we7u Exp $
+ * $Id: maps.h,v 1.75 2012/11/01 18:57:19 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2006  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -100,19 +100,19 @@ typedef struct {
     unsigned long y_lat;
 } tiepoint;
 
-extern void recompute_lat_long(void);
-
 void draw_point(Widget w,
                  unsigned long x1,
                  unsigned long y1,
                  GC gc,
-                 Pixmap which_pixmap);
+                 Pixmap which_pixmap,
+                 int skip_duplicates);
 
 void draw_point_ll(Widget w,
                  float y1,
                  float x1,
                  GC gc,
-                 Pixmap which_pixmap);
+                 Pixmap which_pixmap,
+                 int skip_duplicates);
 
 void draw_vector(Widget w,
                  unsigned long x1,
@@ -120,7 +120,8 @@ void draw_vector(Widget w,
                  unsigned long x2,
                  unsigned long y2,
                  GC gc,
-                 Pixmap which_pixmap);
+                 Pixmap which_pixmap,
+                 int skip_duplicates);
 
 void draw_vector_ll(Widget w,
                  float y1,
@@ -128,13 +129,14 @@ void draw_vector_ll(Widget w,
                  float y2,
                  float x2,
                  GC gc,
-                 Pixmap which_pixmap);
+                 Pixmap which_pixmap,
+                 int skip_duplicates);
 
 char *get_map_ext (char *filename);
 char *get_map_dir (char *fullpath);
 void load_auto_maps(Widget w, char *dir);
 void load_maps(Widget w);
-void fill_in_new_alert_entries(Widget w, char *dir);
+void fill_in_new_alert_entries(void);
 void load_alert_maps(Widget w, char *dir);
 void  index_update_xastir(char *filename, unsigned long bottom, unsigned long top, unsigned long left, unsigned long right, int default_map_layer);
 void  index_update_ll(char *filename, double bottom, double top, double left, double right, int default_map_layer);
@@ -159,8 +161,7 @@ extern int map_visible (unsigned long bottom_map_boundary,
 extern int map_visible_lat_lon (double f_bottom_map_boundary,
                                 double f_top_map_boundary,
                                 double f_left_map_boundary,
-                                double f_right_map_boundary,
-                                char *error_message);
+                                double f_right_map_boundary);
 extern int map_inside_viewport_lat_lon(double map_min_y,
                                        double map_max_y,
                                        double map_min_x,
@@ -177,38 +178,44 @@ extern int print_auto_rotation;
 extern int print_auto_scale;
 extern int print_in_monochrome;
 extern int print_invert;
+extern char printer_program[MAX_FILENAME+1];
+extern char previewer_program[MAX_FILENAME+1];
 
 extern int  gnis_locate_place(Widget w, char *name, char *state,
 char *county, char *quad, char* type, char *filename, int
 follow_case, int get_match, char match_array_name[50][200], long
 match_array_lat[50], long match_array_long[50]);
 
+extern int  pop_locate_place(Widget w, char *name, char *state,
+char *county, char *quad, char* type, char *filename, int
+follow_case, int get_match, char match_array_name[50][200], long
+match_array_lat[50], long match_array_long[50]);
+
 
 extern void maps_init(void);
 enum map_onscreen_enum {MAP_NOT_VIS=0,MAP_IS_VIS,MAP_NOT_INDEXED};
 extern enum map_onscreen_enum map_onscreen(long left, long right, long top, long bottom, int checkpercentage);
 extern enum map_onscreen_enum map_onscreen_index(char *filename);
 extern time_t last_snapshot;
+extern time_t last_kmlsnapshot;
 extern int snapshot_interval;
 
 extern int grid_size;
 
 #if !defined(NO_GRAPHICS)
-  #if defined(HAVE_IMAGEMAGICK)
+  #if defined(HAVE_MAGICK)
     extern float imagemagick_gamma_adjust;
-  #endif    // HAVE_IMAGEMAGICK
+  #endif    // HAVE_MAGICK
 #endif  // NO_GRAPHICS
 
 extern float raster_map_intensity;
  
-extern void Print_properties(Widget widget, XtPointer clientData, XtPointer callData);
+extern void Print_Postscript(Widget widget, XtPointer clientData, XtPointer callData);
 
 extern void map_plot (Widget w, long max_x, long max_y, long x_long_cord, long y_lat_cord, unsigned char color, long object_behavior, int destination_pixmap, int draw_filled); 
 
 extern void map_gdal_init(void);
 
-extern int check_trans (XColor c, unsigned long c_trans_color);
-
 // A struct to pass down in to map driver functions so they can have 
 // driver-specific flags.  Most drivers won't care about any (or even all)
 // of the flags, but this way we can just pass a single pointer rather than
