@@ -1,9 +1,9 @@
 /*
- * $Id: messages.c,v 1.64 2006/04/24 19:21:42 we7u Exp $
+ * $Id: messages.c,v 1.80 2012/09/23 16:19:22 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2006  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -63,6 +63,7 @@
 #include "messages.h"
 #include "util.h"
 #include "interface.h"
+#include "xa_config.h"
 
 // Must be last include file
 #include "leak_detection.h"
@@ -100,7 +101,13 @@ begin_critical_section(&send_message_dialog_lock, "messages.c:clear_message_wind
         mw[i].send_message_dialog = (Widget)NULL;
         mw[i].to_call_sign[0] = '\0';
         mw[i].send_message_call_data = (Widget)NULL;
-        mw[i].send_message_message_data = (Widget)NULL;
+        mw[i].D700_mode = (Widget)NULL;
+        mw[i].D7_mode = (Widget)NULL;
+        mw[i].HamHUD_mode = (Widget)NULL;
+        mw[i].message_data_line1 = (Widget)NULL;
+        mw[i].message_data_line2 = (Widget)NULL;
+        mw[i].message_data_line3 = (Widget)NULL;
+        mw[i].message_data_line4 = (Widget)NULL;
         mw[i].send_message_text = (Widget)NULL;
     }
 
@@ -210,20 +217,28 @@ void group_build_list(char *filename) {
 
 
 
-static int group_active(char *from) {
+int group_active(char *from) {
     static struct stat current_group_stat;
     struct stat group_stat;
     static char altgroup[10];
+    char group_data_path[MAX_VALUE];
+
+    get_user_base_dir(group_data_file, group_data_path, 
+                      sizeof(group_data_path));
 
     (void)remove_trailing_spaces(from);
 
     // If we cycle to/from special group or file changes, rebuild group list.
-    if ((!stat(group_data_file, &group_stat)
+    if ((!stat( group_data_path, &group_stat )
             && (current_group_stat.st_size != group_stat.st_size
                 || current_group_stat.st_mtime != group_stat.st_mtime
-                || current_group_stat.st_ctime != group_stat.st_ctime))
-                || (altgroup && strcasecmp(altgroup, VERSIONFRM))) {
-        group_build_list(group_data_file);
+                || current_group_stat.st_ctime != group_stat.st_ctime))) {
+
+// altgroup equates to "address of altgroup" which always evaluates
+// as true.  Commenting it out of the conditional.  --we7u.
+//                || (altgroup && strcasecmp(altgroup, VERSIONFRM))) {
+
+        group_build_list( group_data_path );
         current_group_stat = group_stat;
         xastir_snprintf(altgroup,sizeof(altgroup),"%s",VERSIONFRM);
     }
@@ -291,8 +306,8 @@ int check_popup_window(char *from_call_sign, int group) {
 
 //fprintf(stderr,"\tcheck_popup_window()\n");
 
-    ret =- 1;
-    found =- 1;
+    ret = -1;
+    found = -1;
 
 begin_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window" );
 
@@ -334,7 +349,7 @@ end_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window"
 
 begin_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window2" );
 
-        i=-1;
+        i= -1;
         for (j=0; j<MAX_MESSAGE_WINDOWS; j++) {
             if (!mw[j].send_message_dialog) {
                 i=j;
@@ -344,7 +359,7 @@ begin_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window
 
 end_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window2" );
 
-        if (i!=-1) {
+        if (i!= -1) {
 
             if (group == 1) {
                 temp1[0] = '*';
@@ -356,9 +371,11 @@ end_critical_section(&send_message_dialog_lock, "messages.c:check_popup_window2"
 
             strncat(temp1,
                 from_call_sign,
-                sizeof(temp1) - strlen(temp1));
+                sizeof(temp1) - 1 - strlen(temp1));
 
-            Send_message(appshell, temp1, NULL);
+            if (!disable_all_popups) {
+                Send_message(appshell, temp1, NULL);
+            }
 
             update_messages(1);
 
@@ -564,8 +581,9 @@ end_critical_section(&send_message_dialog_lock, "messages.c:clear_outgoing_messa
 // sending the sequence number in Base-?? format in order to get
 // more range from the 2-character variable.
 //
-void bump_message_counter(void) {
+int bump_message_counter(char *message_counter) {
 
+    int bump_warning = 0;
     message_counter[2] = '\0';  // Terminate at 2 chars
 
     // Increment the least significant digit
@@ -592,8 +610,9 @@ void bump_message_counter(void) {
 
     if (message_counter[0] == '{') {
         message_counter[0] = '0';
-        message_counter[0]++;
+        bump_warning = 1;
     }
+    return bump_warning;
 }
 
 
@@ -609,13 +628,15 @@ void bump_message_counter(void) {
 //
 void output_message(char *from, char *to, char *message, char *path) {
     int ok,i,j;
-    char message_out[MAX_MESSAGE_OUTPUT_LENGTH+1];
+    char message_out[MAX_MESSAGE_OUTPUT_LENGTH+1+5+1]; // +'{' +msg_id +terminator
     int last_space, message_ptr, space_loc;
     int wait_on_first_ack;
     int error;
     long record;
 
 
+//fprintf(stderr,"output_message:%s\n", message);
+
     message_ptr=0;
     last_space=0;
     ok=0;
@@ -636,22 +657,28 @@ void output_message(char *from, char *to, char *message, char *path) {
         // processed into APRS messages.  Break at a space character
         // if possible.
         //
-        for (j=0;j<MAX_MESSAGE_OUTPUT_LENGTH;j++) {
+        for (j=0; j<MAX_MESSAGE_OUTPUT_LENGTH; j++) {
+
             if(message[j+message_ptr] != '\0') {
+
                 if(message[j+message_ptr]==' ') {
                     last_space=j+message_ptr+1;
                     space_loc=j;
                 }
-                if ((j+1)!=MAX_MESSAGE_OUTPUT_LENGTH) {
+
+                if (j!=MAX_MESSAGE_OUTPUT_LENGTH) {
                     message_out[j]=message[j+message_ptr];
                     message_out[j+1] = '\0';
-                } else {
+                }
+                else {
+
                     if(space_loc!=0)
                         message_out[space_loc] = '\0';
                     else
                         last_space=j+message_ptr;
                 }
-            } else {
+            }
+            else {
                 j=MAX_MESSAGE_OUTPUT_LENGTH+1;
                 last_space=strlen(message)+1;
             }
@@ -662,7 +689,12 @@ void output_message(char *from, char *to, char *message, char *path) {
         if (debug_level & 2)
             fprintf(stderr,"MESSAGE <%s> %d %d\n",message_out,message_ptr,last_space);
 
-        message_ptr=last_space;
+        if (j >= MAX_MESSAGE_OUTPUT_LENGTH) {
+            message_ptr = MAX_MESSAGE_OUTPUT_LENGTH;
+        }
+        else {
+            message_ptr=last_space;
+        }
 
         /* check for others in the queue */
         wait_on_first_ack=0;
@@ -682,7 +714,8 @@ void output_message(char *from, char *to, char *message, char *path) {
                 ok=1;
 
                 // Increment the message sequence ID variable
-                bump_message_counter();
+                if (bump_message_counter(message_counter)) 
+		   fprintf(stderr, "!WARNING!: Wrap around Message Counter");
 
 
 // Note that Xastir's messaging can lock up if we do a rollover and
@@ -933,7 +966,7 @@ void transmit_message_data(char *to, char *message, char *path) {
 // Create a struct to hold the delayed ack's.
 typedef struct _delayed_ack_record {
     char to_call_sign[MAX_CALLSIGN+1];
-    char message_line[MAX_MESSAGE_OUTPUT_LENGTH+1];
+    char message_line[MAX_MESSAGE_OUTPUT_LENGTH+1+5+1];
     char path[200];
     time_t active_time;
     struct _delayed_ack_record *next;
@@ -1202,7 +1235,7 @@ void check_and_transmit_messages(time_t time) {
                             //
                             // Change all messages to that callsign
                             // to match the new path.
-                            // 
+                            //
                             change_path_outgoing_messages_to(to_call,new_path);
                         }
 
@@ -1340,7 +1373,7 @@ void clear_acked_message(char *from, char *to, char *seq) {
     //lowest=100000;
     // Highest Base-90 2-char string
     xastir_snprintf(lowest,sizeof(lowest),"zz");
-    found=-1;
+    found= -1;
     for (i=0; i<MAX_OUTGOING_MESSAGES;i++) {
         if (message_pool[i].active==MESSAGE_ACTIVE) {
 
@@ -1385,7 +1418,7 @@ void clear_acked_message(char *from, char *to, char *seq) {
                             }
                         }
                         // Release the next message in the queue for transmission
-                        if (found!=-1) {
+                        if (found!= -1) {
                             message_pool[found].wait_on_first_ack=0;
                         }
                         else {
