@@ -1,6 +1,6 @@
 
-// $Id: shp_hash.h,v 1.5 2006/01/17 21:05:20 we7u Exp $
-// Copyright (C) 2004-2006  The Xastir Group
+// $Id: shp_hash.h,v 1.11 2012/11/01 18:57:19 we7u Exp $
+// Copyright (C) 2004-2012  The Xastir Group
 
 
 #ifndef __XASTIR_SHP_HASH_H
@@ -32,7 +32,7 @@ void build_rtree(struct Node **root, SHPHandle sHP);
 void destroy_shp_hash(void);
 void empty_shpinfo(shpinfo *si);
 void destroy_shpinfo(shpinfo *si);
-void purge_shp_hash(void);
+void purge_shp_hash(time_t secs_now);
 shpinfo *get_shp_from_hash(char *filename);
 
 #endif // USE_RTREE
