@@ -1,9 +1,9 @@
 /*
- * $Id: track_gui.c,v 1.55 2006/04/24 19:21:42 we7u Exp $
+ * $Id: track_gui.c,v 1.77 2012/09/23 16:19:22 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2006  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +40,7 @@
 #include "lang.h"
 #include "objects.h"
 #include "popup.h"
+#include "fetch_remote.h"
 #include "util.h"
 #include "xa_config.h"
 
@@ -47,6 +48,7 @@
 #include "leak_detection.h"
 
 
+extern XmFontList fontlist1;    // Menu/System fontlist
 
 Widget track_station_dialog = (Widget)NULL;
 Widget track_station_data = (Widget)NULL;
@@ -65,7 +67,7 @@ Widget posit_length_value;
 
 int fetching_findu_trail_now = 0;
 
-int track_station_on;           /* used for tracking stations */
+int track_station_on = 0;       /* used for tracking stations */
 int track_me;
 int track_case;                 /* used for tracking stations */
 int track_match;                /* used for tracking stations */
@@ -88,7 +90,17 @@ void track_gui_init(void)
 {
     init_critical_section( &track_station_dialog_lock );
     init_critical_section( &download_findu_dialog_lock );
-    tracking_station_call[0] = '\0';
+
+    if (temp_tracking_station_call[0] != '\0') {
+        xastir_snprintf(tracking_station_call,
+            sizeof(tracking_station_call),
+            "%s",
+            temp_tracking_station_call);
+        track_station_on = 1;
+    }
+    else {
+        tracking_station_call[0] = '\0';
+    }
 }
 
 
@@ -198,6 +210,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 xmDialogShellWidgetClass, appshell,
                 XmNdeleteResponse, XmDESTROY,
                 XmNdefaultPosition, FALSE,
+                XmNfontList, fontlist1,
                 NULL);
 
         pane = XtVaCreateWidget("Track_station pane",
@@ -228,6 +241,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 XmNrightAttachment, XmATTACH_NONE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         track_station_data = XtVaCreateManagedWidget("Track_station track locate data", 
@@ -250,6 +264,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 XmNrightAttachment,XmATTACH_NONE,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
         track_case_data  = XtVaCreateManagedWidget(langcode("WPUPTSP003"),
@@ -266,6 +281,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         track_match_data  = XtVaCreateManagedWidget(langcode("WPUPTSP004"),
@@ -283,6 +299,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         sep = XtVaCreateManagedWidget("Track_station sep", 
@@ -297,6 +314,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 XmNrightAttachment,XmATTACH_FORM,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         button_ok = XtVaCreateManagedWidget(langcode("WPUPTSP005"),
@@ -316,6 +334,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         button_clear = XtVaCreateManagedWidget(langcode("WPUPTSP006"),
@@ -334,6 +353,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         button_close = XtVaCreateManagedWidget(langcode("UNIOP00003"),
@@ -353,6 +373,7 @@ begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" )
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         XtAddCallback(button_ok, XmNactivateCallback, Track_station_now, track_station_dialog);
@@ -396,6 +417,15 @@ end_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" );
 /***** DOWNLOAD FINDU TRAILS *****/
 
 
+
+// Struct used for passing two parameters to findu_transfer_thread
+typedef struct {
+    char *download_client_ptrs[2];
+} track_ptrs_struct;
+track_ptrs_struct track_ptrs;
+
+
+
 // This is the separate execution thread that fetches the track from
 // findu.  The thread is started up by the Download_trail_now()
 // function below.
@@ -403,7 +433,9 @@ end_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" );
 static void* findu_transfer_thread(void *arg) {
     char *fileimg;
     char *log_filename;
+//    char log_filename_tmp[210];
     char **ptrs;
+    char sys_cmd[128];
 
 
     // Get fileimg and log_filename from parameters
@@ -434,7 +466,8 @@ static void* findu_transfer_thread(void *arg) {
 
         // Reset global "busy" variable
         fetching_findu_trail_now = 0;
-        
+
+        // End the thread
         return(NULL);
     }
 
@@ -450,6 +483,108 @@ static void* findu_transfer_thread(void *arg) {
     // Set permissions on the file so that any user can overwrite it.
     chmod(log_filename, 0666);
 
+#if defined(HAVE_SED)
+    // Add three spaces before each "<br>" and axe the "<br>".  This
+    // is so that Base-91 compressed packets with no comment and no
+    // speed/course/altitude will get decoded ok.  Otherwise the
+    // spaces that are critical to the Base-91 packets won't be
+    // there due to findu.com filtering them out.
+    //
+// "sed -i -e \"s/<br>/   <br>/\" %s",
+    sprintf(sys_cmd,
+        "%s -i -e \"s/<br>/   /\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+//fprintf(stderr,"%s\n", sys_cmd);
+
+// Greater-than symbol '>'
+    sprintf(sys_cmd,
+        "%s -i -e \"s/&gt;/>/\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+
+// Less-than symbol '<'
+    sprintf(sys_cmd,
+        "%s -i -e \"s/&lt;/</\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+
+// Ampersand '&' (A difficult character to escape from the shell!)
+    sprintf(sys_cmd,
+        "%s -i -e \"s/&amp;/\\&/\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+
+// Double-quote symbol '"'
+    sprintf(sys_cmd,
+        "%s -i -e \"s/&quot;/""/\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+
+// Remove whitespace at the start of a line
+// sed 's/^[ \t]*//'
+    sprintf(sys_cmd,
+        "%s -i -e \"s/^[ \t]*//\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+
+// Remove any lines that start with '<'
+    sprintf(sys_cmd,
+        "%s -i -e \"s/^<.*$//\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+
+// Remove any lines that start with '"http'
+    sprintf(sys_cmd,
+        "%s -i -e \"/^\\\"http.*$/d\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+
+// Remove any blank lines from the file
+// sed '/^$/d'
+    sprintf(sys_cmd,
+        "%s -i -e \"/^$/d\" %s",
+        SED_PATH,
+        log_filename);
+    system(sys_cmd);
+#endif  // HAVE_SED
+
+/*
+#ifdef HAVE_HTML2TEXT
+    // Create temp filename
+    snprintf(log_filename_tmp, sizeof(log_filename_tmp), "%s%s",
+        log_filename,
+        ".tmp");
+    // Create html2text command
+    sprintf(sys_cmd,
+        "%s -width 300 -o %s %s",
+        HTML2TEXT_PATH,
+        log_filename_tmp,
+        log_filename);
+    // Convert the newly-downloaded file from html to text format
+    system(sys_cmd);
+    // Rename the file so that we can keep the static char* name
+    // pointing to it, needed for the read_file_ptr code in the main
+    // thread.
+#if defined(HAVE_MV)
+    sprintf(sys_cmd,
+        "%s %s %s",
+        MV_PATH,
+        log_filename_tmp,
+        log_filename);
+    system(sys_cmd);
+#endif  // HAVE_MV
+#endif  // HAVE_HTML2TEXT
+*/
+
     // Here we do the findu stuff, if the findu_flag is set.  Else we do an imagemap.
     // We have the log data we're interested in within the log_filename file.
     // Cause that file to be read by the "File->Open Log File" routine.  HTML
@@ -497,17 +632,20 @@ void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
     char *temp_ptr;
     pthread_t download_trail_thread;
     static XtPointer download_client_data = NULL;
-    static char *download_client_ptrs[2];
 
+    char tmp_base_dir[MAX_VALUE];
+
+    get_user_base_dir("tmp",tmp_base_dir, sizeof(tmp_base_dir));
 
     // If we're already fetching a trail, we shouldn't be calling
     // this callback function.  Get out.
     if (fetching_findu_trail_now)
         return;
 
-    download_client_ptrs[0] = log_filename;
-    download_client_ptrs[1] = fileimg;
-    download_client_data = (XtPointer *)download_client_ptrs;
+    // Pass two parameters to findu_transfer_thread via a struct
+    track_ptrs.download_client_ptrs[0] = log_filename;
+    track_ptrs.download_client_ptrs[1] = fileimg;
+    download_client_data = &track_ptrs;
 
     // Check whether it's ok to do a download currently.
     if (read_file) {
@@ -527,7 +665,7 @@ void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
     xastir_snprintf(log_filename,
         sizeof(log_filename),
         "%s/map.log",
-        get_user_base_dir("tmp"));
+        tmp_base_dir);
 
     // Erase any previously existing local file by the same name.
     // This avoids the problem of having an old tracklog here and
@@ -568,9 +706,19 @@ void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
 // Setting the read_file_ptr to the downloaded file won't do it.
 
 
+
+//        "http://www.findu.com/cgi-bin/rawposit.cgi?call=%s&start=%d&length=%d&time=1", // New, with timestamp
     xastir_snprintf(fileimg, sizeof(fileimg),
-        "http://www.findu.com/cgi-bin/rawposit.cgi?call=%s&start=%d&length=%d",         // Old
-//        "http://www.findu.com/cgi-bin/rawposit.cgi?call=%s&start=%d&length=%d&time=1",// New
+        //
+        // Posits only:
+        // "http://www.findu.com/cgi-bin/rawposit.cgi?call=%s&start=%d&length=%d",
+        //
+        // Posits plus timestamps (we can't handle timestamps yet):
+        // "http://www.findu.com/cgi-bin/rawposit.cgi?call=%s&start=%d&length=%d&time=1", // New, with timestamp
+        // 
+        // Download all packets, not just posits:
+        "http://www.findu.com/cgi-bin/raw.cgi?call=%s&start=%d&length=%d",
+        //
         download_trail_station_call,posit_start,posit_length);
 
     if (debug_level & 1024) {
@@ -633,6 +781,7 @@ void Reset_posit_length_max(Widget w, XtPointer clientData, XtPointer callData)
 void Download_findu_trail( /*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@unused@*/ XtPointer callData) {
     static Widget pane, my_form, button_ok, button_cancel, call, sep;
     Atom delw;
+    XmString x_str;
 
 
     if (!download_findu_dialog) {
@@ -643,6 +792,7 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 xmDialogShellWidgetClass, appshell,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
+                XmNfontList, fontlist1,
                 NULL);
 
         pane = XtVaCreateWidget("Download_findu_trail pane",
@@ -673,6 +823,7 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNrightAttachment, XmATTACH_NONE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         download_trail_station_data = XtVaCreateManagedWidget("download_trail_station_data", 
@@ -695,8 +846,10 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNrightAttachment,XmATTACH_NONE,
                 XmNnavigationType, XmTAB_GROUP,
                 XmNtraversalOn, TRUE,
+                XmNfontList, fontlist1,
                 NULL);
 
+        x_str = XmStringCreateLocalized(langcode("WPUPTSP009"));
         posit_start_value = XtVaCreateManagedWidget("Start of Trail (hrs ago)", 
                 xmScaleWidgetClass, 
                 my_form,
@@ -719,11 +872,18 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNmaximum, MAX_FINDU_START_TIME,
                 XmNshowValue, TRUE,
                 XmNvalue, posit_start,
-                XtVaTypedArg, XmNtitleString, XmRString, langcode("WPUPTSP009"), 22,
+// Note:  Some versions of OpenMotif (distributed with Fedora,
+// perhaps others) don't work properly with XtVaTypedArg() as used
+// here, instead showing blank labels for the Scale widgets.
+//                XtVaTypedArg, XmNtitleString, XmRString, langcode("WPUPTSP009"), 22,
+                XmNtitleString, x_str,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
+        XmStringFree(x_str);
 
+        x_str = XmStringCreateLocalized(langcode("WPUPTSP010"));
         posit_length_value = XtVaCreateManagedWidget("Length of trail (hrs)", 
                 xmScaleWidgetClass, 
                 my_form,
@@ -746,10 +906,16 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNmaximum, MAX_FINDU_DURATION,
                 XmNshowValue, TRUE,
                 XmNvalue, posit_length,
-                XtVaTypedArg, XmNtitleString, XmRString, langcode("WPUPTSP010"), 19,
+// Note:  Some versions of OpenMotif (distributed with Fedora,
+// perhaps others) don't work properly with XtVaTypedArg() as used
+// here, instead showing blank labels for the Scale widgets.
+//                XtVaTypedArg, XmNtitleString, XmRString, langcode("WPUPTSP010"), 19,
+                XmNtitleString, x_str,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
+        XmStringFree(x_str);
 
         sep = XtVaCreateManagedWidget("Download_findu_trail sep", 
                 xmSeparatorGadgetClass,
@@ -763,6 +929,7 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNrightAttachment,XmATTACH_FORM,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         button_ok = XtVaCreateManagedWidget(langcode("WPUPTSP007"),
@@ -782,6 +949,7 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
         if (fetching_findu_trail_now)
             XtSetSensitive(button_ok, FALSE);
@@ -803,6 +971,7 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNtraversalOn, TRUE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         XtAddCallback(button_ok, XmNactivateCallback, Download_trail_now, download_findu_dialog);
