@@ -1,9 +1,9 @@
 /*
- * $Id: view_message_gui.c,v 1.27 2005/01/08 09:54:36 we7u Exp $
+ * $Id: view_message_gui.c,v 1.43 2012/08/25 16:38:29 tvrusso Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <Xm/XmAll.h>
@@ -58,9 +61,11 @@
 #include "main.h"
 #include "util.h"
 
-#ifdef HAVE_DMALLOC
-#include <dmalloc.h>
-#endif  // HAVE_DMALLOC
+// Must be last include file
+#include "leak_detection.h"
+
+
+extern XmFontList fontlist1;    // Menu/System fontlist
 
 Widget All_messages_dialog = NULL;
 Widget view_messages_text = NULL;
@@ -70,6 +75,8 @@ static xastir_mutex All_messages_dialog_lock;
 
 int vm_range;
 int view_message_limit = 10000;
+int Read_messages_packet_data_type = 0; // 1=tnc_only, 2=net_only, 0=tnc&net
+int Read_messages_mine_only = 0;
 
 
 
@@ -96,15 +103,68 @@ void view_message_print_record(Message *m_fill) {
 
     // Make sure it's within our distance range we have set
     distance = (int)distance_from_my_station(m_fill->from_call_sign,temp_my_course);
-    if ((vm_range == 0) || (distance <= vm_range)) {
+
+    if (Read_messages_mine_only
+            || (!Read_messages_mine_only
+                && ( (vm_range == 0) || (distance <= vm_range) ) ) ) {
  
+        // Check that it's coming from the correct type of interface
+        // Compare Read_messages_packet_data_type against the port
+        // type associated with data_port to determine whether or
+        // not to display it.
+        //
+        // I = Internet
+        // L = Local
+        // T = TNC
+        // F = File
+        //
+        switch (Read_messages_packet_data_type) {
+
+            case 2:     // Display NET data only
+                // if not network_interface, return
+                if (m_fill->data_via != 'I')
+                    return; // Don't display it
+                break;
+
+            case 1:     // Display TNC data only
+                // if not local_tnc_interface, return
+                if (m_fill->data_via != 'T')
+                    return; // Don't display it
+                break;
+
+            case 0:     // Display both TNC and NET data
+            default:
+                break;
+        }
+
+        // Check for my stations only if set
+        if (Read_messages_mine_only) {
+            char short_call[MAX_CALLSIGN];
+            char *p;
+
+            xastir_snprintf(short_call,
+                sizeof(short_call),
+                "%s",
+                my_callsign);
+            if ( (p = index(short_call,'-')) ) {
+                *p = '\0';  // Terminate it
+            }
+
+            if (!strstr(m_fill->call_sign, short_call)
+                    && !strstr(m_fill->from_call_sign, short_call)) {
+                return;
+            }
+        }
+
         if ((temp = malloc((size_t)my_size)) == NULL)
             return;
 
-        sprintf(temp,"%-9s>%-9s seq:%5s type:%c :%s\n",
+        sprintf(temp,"%-9s>%-9s %s:%5s %s:%c :%s\n",
             m_fill->from_call_sign,
             m_fill->call_sign,
+            langcode("WPUPMSB013"),
             m_fill->seq,
+            langcode("WPUPMSB014"),
             m_fill->type,
             m_fill->message_line);
 
@@ -156,10 +216,62 @@ void all_messages(char from, char *call_sign, char *from_call, char *message) {
     XmTextPosition drop_ptr;
 
 
-    if ((temp = malloc((size_t)my_size)) == NULL)
-        return;
+    if (Read_messages_mine_only
+            || (!Read_messages_mine_only
+                && ((vm_range == 0)
+                    || ((int)distance_from_my_station(call_sign,temp_my_course) <= vm_range)) ) ) {
+
+        // Check that it's coming from the correct type of interface
+        // Compare Read_messages_packet_data_type against the port
+        // type associated with data_port to determine whether or
+        // not to display it.
+        //
+        // I = Internet
+        // L = Local
+        // T = TNC
+        // F = File
+        //
+        switch (Read_messages_packet_data_type) {
+
+            case 2:     // Display NET data only
+                // if not network_interface, return
+                if (from != 'I')
+                    return; // Don't display it
+                break;
+
+            case 1:     // Display TNC data only
+                // if not local_tnc_interface, return
+                if (from != 'T')
+                    return; // Don't display it
+                break;
+
+            case 0:     // Display both TNC and NET data
+            default:
+                break;
+        }
+
+        // Check for my stations only if set
+        if (Read_messages_mine_only) {
+            char short_call[MAX_CALLSIGN];
+            char *p;
+
+            xastir_snprintf(short_call,
+                sizeof(short_call),
+                "%s",
+                my_callsign);
+            if ( (p = index(short_call,'-')) ) {
+                *p = '\0';  // Terminate it
+            }
+
+            if (!strstr(call_sign, short_call)
+                    && !strstr(from_call, short_call)) {
+                return;
+            }
+        }
+
+        if ((temp = malloc((size_t)my_size)) == NULL)
+            return;
 
-    if ((vm_range == 0) || ((int)distance_from_my_station(call_sign,temp_my_course) <= vm_range)) {
         if (strlen(message)>95) {
             xastir_snprintf(data1,
                 sizeof(data1),
@@ -181,7 +293,7 @@ void all_messages(char from, char *call_sign, char *from_call, char *message) {
         if (strncmp(call_sign, "java",4) == 0) {
             xastir_snprintf(call_sign,
                 MAX_CALLSIGN+1,
-                "Broadcast");
+                "%s", langcode("WPUPMSB015") );   // Broadcast
             xastir_snprintf(temp,
                 my_size,
                 "%s %s\t%s%s\n",
@@ -192,7 +304,7 @@ void all_messages(char from, char *call_sign, char *from_call, char *message) {
         } else if (strncmp(call_sign, "USER", 4) == 0) {
             xastir_snprintf(call_sign,
                 MAX_CALLSIGN+1,
-                "Broadcast");
+                "%s", langcode("WPUPMSB015") );   // Broadcast
             xastir_snprintf(temp,
                 my_size,
                 "%s %s\t%s%s\n",
@@ -231,8 +343,8 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:all_messag
 end_critical_section(&All_messages_dialog_lock, "view_message_gui.c:all_messages" );
 
         }
+        free(temp);
     }
-    free(temp);
 }
 
 
@@ -277,15 +389,55 @@ void All_messages_change_range( /*@unused@*/ Widget widget, XtPointer clientData
     All_messages_destroy_shell(widget, clientData, callData);
     view_all_messages(widget, clientData, callData); 
 }
+
  
 
 
 
+void  Read_messages_packet_toggle( /*@unused@*/ Widget widget, XtPointer clientData, XtPointer callData) {
+    char *which = (char *)clientData;
+    XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;
+
+    if(state->set)
+        Read_messages_packet_data_type = atoi(which);
+    else
+        Read_messages_packet_data_type = 0;
+}
+
+
+
+
+
+Widget button_range;
+
+
+
+
+
+void  Read_messages_mine_only_toggle( /*@unused@*/ Widget widget, XtPointer clientData, XtPointer callData) {
+    char *which = (char *)clientData;
+    XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;
+
+    if(state->set) {
+        Read_messages_mine_only = atoi(which);
+        XtSetSensitive(vm_dist_data, FALSE);
+    }
+    else {
+        Read_messages_mine_only = 0;
+        XtSetSensitive(vm_dist_data, TRUE);
+    }
+}
+
+
+
+
 
 void view_all_messages( /*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@unused@*/ XtPointer callData) {
-    Widget pane, my_form, button_range, button_close, dist, dist_units;
+    Widget pane, my_form, button_close, dist, dist_units;
+    Widget option_box, tnc_data, net_data, tnc_net_data,
+        read_mine_only_button;
     unsigned int n;
-#define NCNT 21
+#define NCNT 50
 #define IncN(n) if (n< NCNT) n++; else fprintf(stderr, "Oops, too many arguments for array!\a")
     Arg args[NCNT];
     Atom delw;
@@ -296,10 +448,10 @@ void view_all_messages( /*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData
 begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_messages" );
 
         All_messages_dialog = XtVaCreatePopupShell(langcode("AMTMW00001"),
-                xmDialogShellWidgetClass,
-                Global.top,
-                XmNdeleteResponse,XmDESTROY,
+                xmDialogShellWidgetClass, appshell,
+                XmNdeleteResponse, XmDESTROY,
                 XmNdefaultPosition, FALSE,
+                XmNfontList, fontlist1,
                 NULL);
 
         pane = XtVaCreateWidget("view_all_messages pane",
@@ -331,6 +483,7 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_m
                 XmNtraversalOn, FALSE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         vm_dist_data = XtVaCreateManagedWidget("view_all_messages dist_data", 
@@ -351,6 +504,7 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_m
                 XmNleftWidget, dist,
                 XmNleftOffset, 10,
                 XmNrightAttachment,XmATTACH_NONE,
+                XmNfontList, fontlist1,
                 NULL);
 
         dist_units = XtVaCreateManagedWidget((english_units?langcode("UNIOP00004"):langcode("UNIOP00005")),
@@ -366,6 +520,7 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_m
                 XmNtraversalOn, FALSE,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         button_range = XtVaCreateManagedWidget(langcode("BULMW00003"),
@@ -381,6 +536,7 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_m
                 XmNnavigationType, XmTAB_GROUP,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         XtAddCallback(button_range, XmNactivateCallback, All_messages_change_range, All_messages_dialog);
@@ -398,10 +554,83 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_m
                 XmNnavigationType, XmTAB_GROUP,
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
                 NULL);
 
         XtAddCallback(button_close, XmNactivateCallback, All_messages_destroy_shell, All_messages_dialog);
+
+        n=0;
+        XtSetArg(args[n],XmNforeground, MY_FG_COLOR); n++;
+        XtSetArg(args[n],XmNbackground, MY_BG_COLOR); n++;
+        XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+        XtSetArg(args[n], XmNtopWidget, dist); n++;
+        XtSetArg(args[n], XmNtopOffset, 5); n++;
+        XtSetArg(args[n], XmNbottomAttachment, XmATTACH_NONE); n++;
+        XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+        XtSetArg(args[n], XmNleftOffset, 5); n++;
+        XtSetArg(args[n], XmNrightAttachment, XmATTACH_NONE); n++;
+        XtSetArg(args[n], XmNfontList, fontlist1); n++;
+
+        option_box = XmCreateRadioBox(my_form,
+                "Vew Messages option box",
+                args,
+                n);
+
+        XtVaSetValues(option_box,
+                XmNpacking, XmPACK_TIGHT,
+                XmNorientation, XmHORIZONTAL,
+                NULL);
+
+        tnc_data = XtVaCreateManagedWidget(langcode("WPUPDPD002"),
+                xmToggleButtonGadgetClass,
+                option_box,
+                MY_FOREGROUND_COLOR,
+                MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
+                NULL);
+
+        XtAddCallback(tnc_data,XmNvalueChangedCallback,Read_messages_packet_toggle,"1");
+
+        net_data = XtVaCreateManagedWidget(langcode("WPUPDPD003"),
+                xmToggleButtonGadgetClass,
+                option_box,
+                MY_FOREGROUND_COLOR,
+                MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
+                NULL);
+
+        XtAddCallback(net_data,XmNvalueChangedCallback,Read_messages_packet_toggle,"2");
+
+        tnc_net_data = XtVaCreateManagedWidget(langcode("WPUPDPD004"),
+                xmToggleButtonGadgetClass,
+                option_box,
+                MY_FOREGROUND_COLOR,
+                MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
+                NULL);
+
+        XtAddCallback(tnc_net_data,XmNvalueChangedCallback,Read_messages_packet_toggle,"0");
+
+        read_mine_only_button = XtVaCreateManagedWidget(langcode("WPUPDPD008"),
+                xmToggleButtonGadgetClass,
+                my_form,
+                XmNvisibleWhenOff, TRUE,
+                XmNindicatorSize, 12,
+                XmNtopAttachment, XmATTACH_WIDGET,
+                XmNtopWidget, dist,
+                XmNtopOffset, 10,
+                XmNbottomAttachment, XmATTACH_NONE,
+                XmNleftAttachment, XmATTACH_WIDGET,
+                XmNleftWidget, option_box,
+                XmNleftOffset, 20,
+                XmNrightAttachment, XmATTACH_NONE,
+                MY_FOREGROUND_COLOR,
+                MY_BACKGROUND_COLOR,
+                XmNfontList, fontlist1,
+                NULL);
  
+        XtAddCallback(read_mine_only_button,XmNvalueChangedCallback,Read_messages_mine_only_toggle,"1");
+
         n=0;
         XtSetArg(args[n], XmNrows, 15); IncN(n);
         XtSetArg(args[n], XmNcolumns, 85); IncN(n);
@@ -416,8 +645,8 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_m
         XtSetArg(args[n], XmNselectionPolicy, XmMULTIPLE_SELECT); IncN(n);
         XtSetArg(args[n], XmNcursorPositionVisible, FALSE); IncN(n);
         XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); IncN(n);
-        XtSetArg(args[n], XmNtopWidget, dist); IncN(n);
-        XtSetArg(args[n], XmNtopOffset, 20); IncN(n);
+        XtSetArg(args[n], XmNtopWidget, option_box); IncN(n);
+        XtSetArg(args[n], XmNtopOffset, 5); IncN(n);
         XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); IncN(n);
         XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); IncN(n);
         XtSetArg(args[n], XmNleftOffset, 5); IncN(n);
@@ -425,6 +654,7 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_m
         XtSetArg(args[n], XmNrightOffset, 5); IncN(n);
         XtSetArg(args[n], XmNforeground, MY_FG_COLOR); IncN(n);
         XtSetArg(args[n], XmNbackground, MY_BG_COLOR); IncN(n);
+        XtSetArg(args[n], XmNfontList, fontlist1); n++;
  
         view_messages_text = XmCreateScrolledText(my_form,
                 "view_all_messages text",
@@ -442,6 +672,34 @@ begin_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_m
         sprintf(temp,"%d",vm_range);
         XmTextFieldSetString(vm_dist_data,temp);
 
+        switch (Read_messages_packet_data_type) {
+            case(0):
+                XmToggleButtonSetState(tnc_net_data,TRUE,FALSE);
+                break;
+
+            case(1):
+                XmToggleButtonSetState(tnc_data,TRUE,FALSE);
+                break;
+
+            case(2):
+                XmToggleButtonSetState(net_data,TRUE,FALSE);
+                break;
+
+            default:
+                XmToggleButtonSetState(tnc_net_data,TRUE,FALSE);
+                break;
+        }
+
+        if (Read_messages_mine_only) {
+            XmToggleButtonSetState(read_mine_only_button,TRUE,FALSE);
+            XtSetSensitive(vm_dist_data, FALSE);
+        }
+        else {
+            XmToggleButtonSetState(read_mine_only_button,FALSE,FALSE);
+            XtSetSensitive(vm_dist_data, TRUE);
+        }
+
+        XtManageChild(option_box);
         XtManageChild(view_messages_text);
         XtVaSetValues(view_messages_text, XmNbackground, colors[0x0f], NULL);
         XtManageChild(my_form);
@@ -471,3 +729,4 @@ end_critical_section(&All_messages_dialog_lock, "view_message_gui.c:view_all_mes
     }
 }
 
+
