@@ -1,8 +1,8 @@
 /*
- * $Id: x_spider.h,v 1.6 2005/01/08 09:46:48 we7u Exp $
+ * $Id: x_spider.h,v 1.16 2012/11/01 18:57:19 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
- * Copyright (C) 2003-2005  The Xastir Group
+ * Copyright (C) 2003-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,20 +24,25 @@
 #ifndef __XASTIR_SERVER_H
 #define __XASTIR_SERVER_H
 
-#include <pthread.h>
 //#include "xastir.h"
 
 
 #define NET_CONNECT_TIMEOUT 20
 #define SERV_TCP_PORT       2023
+#define SERV_UDP_PORT       2023
+
 
 char *pname;
-extern int pipe_xastir_to_server;
-extern int pipe_server_to_xastir;
+extern int pipe_xastir_to_tcp_server;
+extern int pipe_tcp_server_to_xastir;
+extern int pipe_xastir_to_udp_server;
+extern int pipe_udp_server_to_xastir;
 
 extern int writen(register int fd, register char *ptr, register int nbytes);
 extern int readline(register int fd, register char *ptr, register int maxlen);
-extern int Fork_server(int argc, char *argv[], char *envp[]);
+extern int Fork_TCP_server(int argc, char *argv[], char *envp[]);
+extern int Fork_UDP_server(int argc, char *argv[], char *envp[]);
+
 
 #endif /* XASTIR_SERVER_H */
 
