@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: xastir.h,v 1.45 2005/07/24 02:29:57 we7u Exp $
+ * $Id: xastir.h,v 1.65 2012/11/01 18:57:19 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2012  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,55 +28,6 @@
 #define _XASTIR_H
 
 
-// If libgc is installed, uncomment this next line to enable memory
-// leak detection:
-#define DETECT_MEMORY_LEAKS
-
-
-// Defines for including the libgc garbage collection library.
-// This enables automatic garbage collection of unused memory,
-// very similar to the garbage collection in Java.  Get libgc from
-// here:  http://www.hpl.hp.com/personal/Hans_Boehm/gc/
-//
-// This will cause stats to be printed every 60 seconds, 'cuz we
-// call GC_collect via a macro from UpdateTime() once per minute:
-// export GC_PRINT_STATS=1; xastir &
-//
-// Compile libgc with this option for more debugging output.  I
-// didn't do so:  --enable-full_debug 
-//
-// If we enable these thread options, Xastir won't link with the
-// library.  Since we don't allocate dynamic memory in the child
-// threads anyway, skip them.
-// --enable-threads=posix --enable-thread-local-alloc --enable-parallel-mark
-//
-// Call GC_gcollect at appropriate points to check for leaks.  We do
-// this via the CHECK_LEAKS macro called from main.c:UpdateTime.
-//
-#ifdef HAVE_GC_H
-  #ifdef HAVE_LIBGC
-
-    // We use this define to enable code in *.c files
-    #define USING_LIBGC
-
-    // Set up for threads
-//    #define GC_THREADS
-
-    // Ask for more debugging
-    #define GC_DEBUG
-
-    #include <malloc.h>
-    #include <gc.h>
-    #define malloc(n) GC_MALLOC(n)
-    #define calloc(m,n) GC_MALLOC((m)*(n))
-    #define free(p) GC_FREE(p)
-    #define realloc(p,n) GC_REALLOC((p),(n))
-    #define CHECK_LEAKS() GC_gcollect()
-
-  #endif    // HAVE_LIBGC
-#endif  // HAVE_GC_H
- 
-
 
 // Macros that help us avoid warnings on 64-bit CPU's.
 // Borrowed from the freeciv project (also a GPL project).
@@ -106,7 +57,7 @@
 #define MY_BG_COLOR             colors[0xff]
 #define MY_BACKGROUND_COLOR     XmNbackground,colors[0xff]
 
-
+// Latitude and longitude string formats.
 #define CONVERT_HP_NORMAL       0
 #define CONVERT_HP_NOSP         1
 #define CONVERT_LP_NORMAL       2
@@ -115,11 +66,15 @@
 #define CONVERT_UP_TRK          5
 #define CONVERT_DMS_NORMAL      6
 #define CONVERT_VHP_NOSP        7
+#define CONVERT_DMS_NORMAL_FORMATED      8
+#define CONVERT_HP_NORMAL_FORMATED       9
 
 #ifndef M_PI                      /* if not defined in math.h */
 #define M_PI 3.14159265358979323846
 #endif  // M_PI
 
+#define SPEECH_TEST_STRING      "Greeteengz frum eggzaster"
+
 /* GLOBAL DEFINES */
 extern char dangerous_operation[200];
 extern GC gc;
@@ -131,14 +86,9 @@ extern Pixmap  pixmap_25pct_stipple;
 extern Pixmap  pixmap_13pct_stipple;
 extern Pixmap  pixmap_wx_stipple;
 
-
-typedef struct XastirGlobal {
-    Widget  top;    // top level shell
-} XastirGlobal;
-
-extern XastirGlobal Global;
 extern Widget appshell;
 
+
 extern int wait_to_redraw;
 /*extern char my_callsign[MAX_CALLSIGN+1];*/
 
@@ -153,30 +103,44 @@ extern void Send_message(Widget w, XtPointer clientData, XtPointer callData);
 extern void create_gc(Widget w);
 
 extern void Station_info(Widget w, XtPointer clientData, XtPointer calldata);
-extern void Window_Quit(Widget w, XtPointer client, XtPointer call);
 
 extern void fix_dialog_size(Widget w);
 extern void fix_dialog_vsize(Widget w);
 
 extern int debug_level;
 extern GC gc;
-extern int colors[];
-extern long mid_x_long_offset;
-extern long mid_y_lat_offset;
-extern long x_long_offset;
-extern long y_lat_offset;
-extern long scale_x;            // x scaling in 1/100 sec per pixel
-extern long scale_y;            // y scaling in 1/100 sec per pixel
-extern float f_center_latitude;
-extern float f_center_longitude;
+extern Pixel colors[];
+
+
+extern float f_center_longitude;   // Floating point map center longitude
+extern float f_center_latitude;    // Floating point map center latitude
+extern float f_NW_corner_longitude;// longitude of NW corner
+extern float f_NW_corner_latitude; // latitude of NW corner
+extern float f_SE_corner_longitude;// longitude of SE corner
+extern float f_SE_corner_latitude; // latitude of SE corner
+
+extern long center_longitude;      // Longitude at center of map
+extern long center_latitude;       // Latitude at center of map
+extern long NW_corner_longitude;   // longitude of NW corner
+extern long NW_corner_latitude;    // latitude of NW corner
+extern long SE_corner_longitude;   // longitude of SE corner
+extern long SE_corner_latitude;    // latitude of SE corner
+
+extern long scale_x;               // x scaling in 1/100 sec per pixel
+extern long scale_y;               // y scaling in 1/100 sec per pixel
+
+
 extern long screen_width;
 extern long screen_height;
+extern Position screen_x_offset;
+extern Position screen_y_offset;
 extern int long_lat_grid;
 //extern Pixmap  pixmap;
 //extern Pixmap  pixmap_final;
 //extern Pixmap  pixmap_alerts;
 extern int map_color_levels;
 extern int map_labels;
+extern int map_lock_pan_zoom;
 extern int map_auto_maps;
 extern int auto_maps_skip_raster;
 extern time_t sec_remove;
@@ -188,16 +152,21 @@ extern int redraw_on_new_data;
 extern int tiger_flag;
 extern int index_maps_on_startup;
 #define MAX_LABEL_FONTNAME 256
-#define FONT_TINY 0
-#define FONT_SMALL 1
-#define FONT_MEDIUM 2
-#define FONT_LARGE 3
-#define FONT_HUGE 4
-#define FONT_MAX 5
+#define FONT_SYSTEM  0
+#define FONT_STATION 1
+#define FONT_TINY    2
+#define FONT_SMALL   3
+#define FONT_MEDIUM  4
+#define FONT_LARGE   5
+#define FONT_HUGE    6
+#define FONT_BORDER  7
+#define FONT_ATV_ID  8
+#define FONT_MAX     9
 #define FONT_DEFAULT FONT_MEDIUM
+#define MAX_FONTNAME 256
 extern char rotated_label_fontname[FONT_MAX][MAX_LABEL_FONTNAME];
 
-#ifdef HAVE_IMAGEMAGICK  //N0VH
+#ifdef HAVE_MAGICK  //N0VH
 extern int tiger_show_grid;
 extern int tiger_show_counties;
 extern int tiger_show_cities;
@@ -212,7 +181,7 @@ extern int tiger_show_statehwy;
 extern int tiger_show_water;
 extern int tiger_show_lakes;
 extern int tiger_show_misc;
-#endif  // HAVE_IMAGEMAGICK
+#endif  // HAVE_MAGICK
 
 #ifdef HAVE_LIBGEOTIFF
 extern int DRG_XOR_colors;
